var AWS = require('hws-sdk/lib/core');
var fs = require('fs');
var Task = require("./TaskModel").Task

function DownloadPool() {

}

constructor: function DownloadPool(s3, db) {
    var self = this;
    AWS.SequentialExecutor.call(self);
    this.s3 = s3;
    this.db = db;

    this.serviceHandles = new Map();  //key:task_id, value:  s3.requst/ s3.download
}

//读取并执行任务
DownloadPool.prototype.run = function run(task) {
    var self = this;
    var filePath = task.path;
    var fatherPath = filePath.substring(0, filePath.lastIndexOf("/"));
    fs.exists(fatherPath, function (exists) {
        if (!exists) {
            self.createLocalFolder(fatherPath, function (err) {
                if (err) {
                    global.logger.error('createLocalFolder error', err);
                    task.markError(err);
                    self.finish(task);
                } else {
                    self.download(task);
                }
            });
        }
        else {
            self.download(task);
        }
    });
}

//任务结束，包含正常结束、暂停、失败、取消
DownloadPool.prototype.finish = function finish(task, callback) {
    var self = this;
    var times = 0;
    self.serviceHandles.delete(task.id);
    global.logger.info("downloadpool task finish.....key,stat,errMessage:", task.key, task.stat, task.errMessage)
    global.logger.info("downloadpool task serviceHandles:", self.serviceHandles.size);
    if (task.startTime) {
        times = new Date().getTime() - task.startTime.getTime();
    }
    if (task.stat === "RUNNING") {
        var stat = fs.lstatSync(task.path);
        var isDirectory = stat.isDirectory();
        if (global.isMD5check === "Y" && !isDirectory) {
            var params = task.buildHeadObjectParam();
            s3.headObject(params, function (err, data) {
                if (err) {
                    logger.error("[download] head object to get md5 failed.", err.stack);
                    self.markTaskStat(task, Task.TaskStatEnum.MD5FAILED);
                }
                var MD5 = data.Metadata.md5chksum;
                if (!MD5) {
                    self.markTaskStat(task, Task.TaskStatEnum.SUCCESS);
                } else {
                    task.changeStat(Task.TaskStatEnum.MD5Check);
                    task.computeMD5(function (err, md5, size, task) {
                        if (err) {
                            //do something...
                            logger.error("[download] compute md5 failed.", task.key, task.path);
                            self.markTaskStat(task, Task.TaskStatEnum.MD5FAILED);
                            return;
                        }
                        if (MD5 === md5) {
                            self.markTaskStat(task, Task.TaskStatEnum.MD5SUCCESS);
                        } else {
                            self.markTaskStat(task, Task.TaskStatEnum.MD5FAILED);
                        }
                    })
                }
            });
            global.successLogger.info("download", 'key:', task.key, "size:", task.size, "accountName:", global.currentUserName, "duration:", times, "status:", "Success");
        } else {
            self.markTaskStat(task, Task.TaskStatEnum.SUCCESS);
        }
    }
    else if (task.stat === "PAUSE") {
        self.markTaskStat(task, Task.TaskStatEnum.PAUSE, "onPause");
    }
    else if (task.stat === "FAILED") {
        global.successLogger.info("download", 'key:', task.key, "size:", task.size, "accountName:", global.currentUserName, "duration:", times, "errCode:", task.errMessage, "status:", "Failed");
        if (task.errMessage === 'NetworkingError') {
            self.markTaskStat(task, Task.TaskStatEnum.WAIT, "onWait");
        } else {
            self.markTaskStat(task,Task.TaskStatEnum.FAILED, "failed");
        }
    }
    else if (task.stat === "ABORT") {
        self.db.delById(task.id, function (err, data) {
            if (!err) {
                self.emit("onAbort", [task]);
            }
            else {
                global.logger.error(err)
            }
        });
    }
    if (callback) {
        callback.call();
    }
};

/**
 * 标注task结束后进行持久化，并发送task end事件
 * @param Task
 * @param stat  SUCCESS/MD5SUCCESS/MD5FAILED
 */
DownloadPool.prototype.markTaskStat = function (task, stat, event = "end") {
    var self = this;
    task.changeStat(stat);
    self.db.updateStat(task, function (err, data) {
        if (!err) {
            self.emit(event, [task]);
        }
        else {
            global.logger.error("[download] task finished,but update stat failed.", task.path, task.stat, err)
        }
    });
};


//文件大小为size<=64K
DownloadPool.prototype.downloadLess64kb = function (task) {
    var self = this;
    var out = fs.createWriteStream(task.path)
        .on("open", function (fd) {
            if (fd) {
                if(global.shareDownload){
                    var params = {
                        Bucket : task.bucket,
                        Key : task.key,
                        Policy:global.Policy,
                        AWSAccessKeyId:global.AccessKeyId,
                        Signature:global.Signature,
                    };
                    var req = self.s3.getObjectByUrl(params);
                }else{
                    var params = {Bucket: task.bucket, Key: task.key};
                    var req = self.s3.getObject(params);
                }

                self.serviceHandles.set(task.id, req);
                var loaded = 0;
                var readStream = req.createReadStream()
                    .on('data', function (chunk) {
                        loaded += chunk.length;
                        var info = {loaded: loaded, total: task.size};
                        self.emit("progress", [task, info]);
                    })
                    .on('error', function (err) {
                        global.logger.error('[Download] <64kb error:', err, task.path, task.stat);
                        task.markError(err);
                        fs.unlink(task.path, (err) => {
                        });  //删除
                        out.end();
                    });
                readStream.pipe(out);
            }
            else {
                task.markError("unknow err");
                self.finish(task);
            }
        })
        .on("finish", function (fd) {
            self.finish(task);
        })
        .on('error', function (err) {
            global.logger.error("[download] less 64 writestream:", err);
            task.markError(err);
            self.finish(task);
        });
};

//下载文件
DownloadPool.prototype.download = function download(task) {
    var self = this;
    if(task.path.length > 259 ){
        task.markError("ENOENT");
        self.finish(task);
        return false;
    }
    if (task.size === 0 && task.path.charAt(task.path.length - 1) == '/') {
        //文件大小为0且以文件夹形式
        fs.exists(task.path, function (exists) {
            if (exists) {
                global.logger.info("[download] the path is exist and is folder.", task.path)
                self.finish(task);
            } else {
                fs.mkdir(task.path, function (err) {
                    if (err) {
                        global.logger.error('[download] mkdir error', err, task.path);
                        task.markError(err);
                        self.finish(task);
                    } else {
                        self.finish(task);
                    }
                });
            }
        });
        return;
    }
    if (task.size == 0) {  //...
        this.downloadLess64kb(task)
        return;
    }

    var params = task.buildParam();
    var options = task.buildDownloadOption();
    var headParams = task.buildHeadObjectParam();
    if(global.shareDownload){
        var headParams = {
            Bucket : global.Bucket,
            Key : task.key,
            Policy:global.Policy,
            AWSAccessKeyId:global.AccessKeyId,
            Signature:global.Signature,
            Range: "bytes=" + 0 + "-" + 1
        };
        s3.getObjectByUrl(headParams,function (err,data) {
            if(err){
                global.logger.info("[download]  error:", err)
                task.markError(err);
                self.finish(task);
            }else{
                var download = self.s3.download(params, options).on('httpDownloadProgress', function (evt) {
                    self.emit("progress", [task, evt]);
                });
                self.serviceHandles.set(task.id,download);
                download.send(function (err, data) {
                    if (err) {
                        //如果此回调执行时，download已经改变。则忽略此回调.
                        if (self.serviceHandles.get(task.id) !== download) {
                            global.logger.warn("[download] networkerrorS. and this err must be socket hang up.", err)
                            return;
                        }
                        global.logger.error("[download] >64kb error:", err);
                        task.saveProcessInfo(download);
                        task.markError(err);
                        //用户取消任务,则删除下载的文件
                        if (err.code === "RequestAbortedError") {
                            fs.unlink(task.path, (err) => {
                                logger.info("[download]aborted unlink....", task.path, err);
                        });
                        }
                    }
                    self.finish(task);
                });
            }
        })
    }else{
        s3.headObject(headParams,function (err,data) {
            if(err){
                global.logger.info("[download]  error:", err)
                task.markError(err);
                self.finish(task);
            }else{
                var download = self.s3.download(params, options).on('httpDownloadProgress', function (evt) {
                    self.emit("progress", [task, evt]);
                });
                self.serviceHandles.set(task.id,download);
                download.send(function (err, data) {
                    if (err) {
                        //如果此回调执行时，download已经改变。则忽略此回调.
                        if (self.serviceHandles.get(task.id) !== download) {
                            global.logger.warn("[download] networkerrorS. and this err must be socket hang up.", err)
                            return;
                        }
                        global.logger.error("[download] >64kb error:", err);
                        task.saveProcessInfo(download);
                        task.markError(err);
                        //用户取消任务,则删除下载的文件
                        if (err.code === "RequestAbortedError") {
                            fs.unlink(task.path, (err) => {
                                logger.info("[download]aborted unlink....", task.path, err);
                        });
                        }
                    }
                    self.finish(task);
                });
            }
        })
    }


    global.logger.info("[download]  start:", params, options)


};

/**
 *
 * @param task 此task必须在taskmanager的runningMap中
 */
DownloadPool.prototype.pause = function (task) {
    var serviceHandle = this.serviceHandles.get(task.id);
    if (serviceHandle) {
        if (serviceHandle.pause) {   //如果有pause方法表示为分段下载。
            serviceHandle.pause();
        } else {   //没有<64kb的特殊处理，现在不会走到这个分支，暂时保留一段时间
            task.stat = Task.TaskStatEnum.PAUSE;
            serviceHandle.removeAllListeners('httpDownloadProgress');
            serviceHandle.removeAllListeners('complete');
            serviceHandle.removeAllListeners('error');
            serviceHandle.abort();
        }
        this.serviceHandles.delete(task.id);
    }
}

/**
 *
 * @param task 此task必须在taskmanager的runningMap中
 */
DownloadPool.prototype.abort = function (task) {
    var serviceHandle = this.serviceHandles.get(task.id);
    if (serviceHandle) {
        task.stat = Task.TaskStatEnum.ABORT;
        serviceHandle.abort();
        this.serviceHandles.delete(task.id);
    }
}


//创建本地文件系统文件夹
DownloadPool.prototype.createLocalFolder = function createLocalFolder(path, callback) {

    var fatherPath = path.substring(0, path.lastIndexOf("/", path.length - 2) + 1);
    try {
        fs.exists(fatherPath, function (exists) {
            if (exists) {
                //创建本地文件系统文件夹
                fs.mkdir(path, function (err) {
                    if (callback) {
                        callback.call(err);
                    }
                });
            } else {
                //创建本地文件系统文件夹
                createLocalFolder(fatherPath, function () {
                    fs.mkdir(path, function (err) {
                        if (callback) {
                            callback.call(err);
                        }
                    });
                });

            }
        });
    } catch (e) {
        global.logger.error('createLocalFolder error', e, path)
    }

};

AWS.util.mixin(DownloadPool, AWS.SequentialExecutor);
module.exports = DownloadPool;