var AWS = require('hws-sdk/lib/core');
var fs = require('fs');
var path = require("path");
var obsError = require("./common/ObsError.js");
var CommonUtils = require("./common/CommonUtils.js");
var Task = require("./TaskModel.js").Task;

function BucketPool() {

}

constructor: function BucketPool(s3, db) {
    AWS.SequentialExecutor.call(this);
    this.s3 = s3;
    this.db = db;
    this.serviceHandles = new Map();    //key:task_id, value:  s3.requst/ s3.upload
}


//读取并执行任务
BucketPool.prototype.run = function run(task) {
    var self = this;
    logger.info("[bucketPool] start running task....", task.key, task.type);
    if (task.type === "create") {
        self.createObject(task);
    } else if (task.type === "putObj") {
        self.upload(task);
    } else if (task.type === "delete") {
        var params = {
            Bucket: task.bucket,
            Key: task.key
        };
        self.s3.deleteObject(params, function (err) {
            if (err) {
                global.logger.error('[s3] del obj', err);
                task.stat = "FAILED";
                task.errMessage = err.code;
            }
            self.finish(task);
        });
    } else if (task.type === "copy" || task.type === "move" || task.type === "rename") {
        self.copyMoveObj(task);
    }
};

BucketPool.prototype.createObject = function (task) {
    var self = this;
    var params = {
        Bucket: task.bucket,
        Key: task.key
    };
    if (global.serverEncryption) {
        params.ServerSideEncryption = 'aws:kms';
    }
    self.s3.putObject(params, function (err, data) {
        if (err) {
            global.logger.error('[s3] put obj--->', err);
            task.stat = "FAILED";
            task.errMessage = err.code;
        }

        self.finish(task);
    });
}

/**
 * 拷贝移动具体执行逻辑
 * @param task
 */
BucketPool.prototype.copyMoveObj = function (task) {
    let self = this;
    let ManagedCopy = require("./sdk/managed_copy");
    let managedCopy = new ManagedCopy(self.s3, task);
    self.serviceHandles.set(task.id, managedCopy);
    managedCopy.on("copyProgress", function (evt) {
        self.emit("progress", [managedCopy.task, evt]);
    })
    let promise = managedCopy.process();
    promise.then(() => {
        self.finish(managedCopy.task);
    }).catch((err) => {
        if (self.serviceHandles.get(task.id) !== managedCopy) {
            global.logger.warn("[copy,rename,move] networkerrorS. and this err must be socket hang up.", err);
            return;
        }
        if (err) {
            logger.error("[copy rename move] Exception happen!", managedCopy.task.key, err);
        }
        managedCopy.task.markError(err);
        managedCopy.task.errMessage = obsError(err.code) ? obsError(err.code).description : err.message;
        self.finish(managedCopy.task);
    }).catch((err) => {
        logger.info("[move copy rename]unpredictable failed!!!!!!!!", err);
        self.markTaskStat(task, Task.TaskStatEnum.FAILED, "failed");
    });
}


BucketPool.prototype.dealUploadNameTooLong = function (task) {
    //如果对象KEY大于1023个字节，则直接让任务失败
    var objectNameLength = new Buffer(task.key).length;
    if (objectNameLength > 1023) {
        global.logger.error('[local] put obj error code:KeyTooLongError,message:The absolute path of an object cannot exceed 1023 bytes.');
        task.markError("KeyTooLongError");
        this.finish(task);
        return true;
    }
    return false;
}

BucketPool.prototype.dealUploadSizeZero = function (task) {
    var self = this;
    //文件大小为0，文件夹也走这里
    var params = task.buildUploadParam(false);
    //size为0的没有req句柄，无法暂停取消
    this.s3.putObject(params, function (err, data) {
        if (err) {
            global.logger.error('[s3] put obj failed-->', task, err);
            task.stat = "FAILED";
            task.errMessage = err.code;
        }
        self.finish(task);
    });
}

BucketPool.prototype.compareUpload = function (task) {
        var self = this;
        task.stat = "skip"
        self.finish(task);
}

BucketPool.prototype.uploadLess64KB = function (task) {
    var self = this;
    //文件大小为size<=64K
    fs.readFile(task.path, function (err, body) {
        if (err) {
            global.logger.info("read upload file content failed...", task.path, err)   //基本不会发生
            task.markError(err);
            self.finish(task);
            return;
        }
        try {
            var params = task.buildUploadParam(body);
            var req = this.s3.putObject(params).on('httpUploadProgress', function (evt) {
                if (task.stat === "RUNNING")
                    self.emit("progress", [task, evt]);
            });
            self.serviceHandles.set(task.id, req);
            global.logger.info('[mult uploader] <64 --upload send key param:', task.key, params);
            req.send(function (err, data) {
                if (err) {
                    global.logger.error('[s3]--put obj <64kb-->key stat err:', task.key, task.stat, err);
                    //networkerror可能导致回调2次，一次是根因，还有一次是根因导致的本端socket关闭。如果第2次异常出现时任务已经重新开始执行，则不能再次重试。
                    if (self.serviceHandles.get(task.id) !== req) {
                        global.logger.warn("[putobj] networkerrorS. and this err must be socket hang up.");
                        return;
                    }
                    task.markError(err);
                }
                self.finish(task);
            });
        } catch (err) {
            global.logger.error('[upload pool]--catch-->', err);
            task.markError(err);
            self.finish(task);
        }
    });
}

BucketPool.prototype.uploadMore64KB = function (task) {
    var self = this;
    var params = task.buildUploadParam();
    var options = task.buildUploadOptions();


    global.logger.info('[mult uploader] >64 --upload send>>', task.key, params, options);
    var upload = this.s3.upload(params, options, function (err, data) {
        if (err) {
            global.logger.error('[s3]--put obj >64kb-->key stat err:', task.key, task.stat, err);
            //networkerror可能导致回调2次，一次是根因，还有一次是根因导致的本端socket关闭。
            // 如果第2次异常出现时任务已经重新开始执行，则不能再次重试
            if (self.serviceHandles.get(task.id) !== upload) {
                global.logger.warn("[putobj] networkerrorS. and this err must be socket hang up.", err)
                return;
            }
            task.saveProcessInfo(upload);
            task.markError(err);
        }
        self.finish(task);
    });
    upload.on('successCreateMultipartUpload', function (UploadId) {  //成功创建分段上传uploadid。
        task.UploadId = UploadId;  //保存uploadid
    }).on('httpUploadProgress', function (evt) {
        self.emit("progress", [task, evt]);
    }).on('partComplete', function (part) {
        task.saveProcessInfo(upload);
        self.db.updateStat(task, function (err) {
            if (err) {
                global.logger.error("[upload pool] >64kb db save process failed.", err);
            }
        });
    });
    self.serviceHandles.set(task.id, upload);
}

//上传文件
BucketPool.prototype.upload = function (task) {
    var self = this;
    new Promise((resolve, reject) => {
        fs.stat(task.path, function (err, stats) {
            if (err)
                return reject(err);
            else
                resolve(stats);
        });
    }).then(function (stats) {
        if (self.dealUploadNameTooLong(task)) {
            return null;
        }
        if (stats.size === 0) {
            self.dealUploadSizeZero(task);
            return null;
        }
        return stats;
    }).catch(function (err) {
        //文件不存在
        task.markError(err);
        global.logger.error('[fs]--NoSuchFile-->', task);
        self.finish(task);
    }).then((stats) => {
        if (!stats) {
            return;
        }
    var params = {
        Bucket: task.bucket,
        Key:task.key
    }
    if(task.compare){
        s3.headObject(params,function (err,data) {
            if(data && (data.ContentLength == stats.size)){
                if(Date.parse(data.LastModified) >  Date.parse(stats.mtime)){
                    self.compareUpload(task);
                }else{
                    if (stats.size <= 65536) {
                        self.uploadLess64KB(task);
                        return;
                    }
                    self.uploadMore64KB(task);
                }
            }else{
                if (stats.size <= 65536) {
                    self.uploadLess64KB(task);
                    return;
                }
                self.uploadMore64KB(task);
            }
        })
    }else{
        if (stats.size <= 65536) {
            self.uploadLess64KB(task);
            return;
        }
        self.uploadMore64KB(task);
    }

    }).catch(function (err) {
        global.logger.error("[upload] unpredictable problem find......", err);
    });
};

//任务结束，包含正常结束、暂停、失败、取消
BucketPool.prototype.finish = function finish(task, callback) {
    var self = this;
    self.serviceHandles.delete(task.id);
    global.logger.info("uploadbucketPool task finish.....key,stat,errMessage:", task.key, task.stat, task.errMessage)
    global.logger.info("uploadbucketPool task serviceHandles:", self.serviceHandles.size);
    var times = 0;
    if (task.startTime) {
        times = new Date().getTime() - task.startTime.getTime();
    }
    if (task.stat === "RUNNING") {
        if (task.type === "putObj" && global.isMD5check === 'Y') {
            var stat = fs.lstatSync(task.path);
            var isDirectory = stat.isDirectory();
            if (isDirectory) {
                self.markTaskStat(task, Task.TaskStatEnum.SUCCESS)
            } else {
                task.changeStat(Task.TaskStatEnum.MD5Check);
                task.computeMD5(function (error, md5num, size, task) {
                    if (error) {
                        self.markTaskStat(task, Task.TaskStatEnum.MD5FAILED, "failed");
                        return;
                    }
                    var headParams = task.buildHeadObjectParam();
                    s3.headObject(headParams, function (err, data) {
                        if (data.ContentLength == size) {
                            var ContentType = data.ContentType;
                            var params = task.buildReplaceObjectParam({HeaderMD5: md5num,ContentType:ContentType});
                            if (global.serverEncryption) {
                                params.ServerSideEncryption = 'aws:kms';
                            }
                            s3.copyObject(params, function (err, data) {
                                if(task.movePath){
                                    self.move(task)
                                }
                                self.markTaskStat(task, Task.TaskStatEnum.MD5SUCCESS);
                            })
                        } else {
                            self.markTaskStat(task, Task.TaskStatEnum.MD5FAILED, "failed");
                        }
                    })
                })
                global.successLogger.info("upload", 'key:', task.key, "size:", task.size, "accountName:", global.currentUserName, "duration:", times, "status:", "MD5Success");
            }
        } else {
            if(task.movePath){
               self.move(task)
            }
            self.markTaskStat(task, Task.TaskStatEnum.SUCCESS);
        }
    } else if (task.stat === "PAUSE") {
        self.markTaskStat(task, Task.TaskStatEnum.PAUSE, "onPause");

    } else if (task.stat === "FAILED") {
        if (task.type === "putObj") {
            global.successLogger.info("upload", 'key:', task.key, "size:", task.size, "accountName:", global.currentUserName, "duration:", times, "errCode:", task.errMessage, "status:", "Failed");
        } else if (task.type === "delete") {
            global.successLogger.info("delete", 'key:', task.key, "size:", task.size, "accountName:", global.currentUserName, "duration:", times, "errCode:", task.errMessage, "status:", "Failed");
        }
        if (task.errMessage === 'NetworkingError') {
            self.markTaskStat(task, Task.TaskStatEnum.WAIT, "onWait");
        } else {
            self.markTaskStat(task, Task.TaskStatEnum.FAILED, "failed");
        }
    } else if (task.stat === "WAIT") {
        self.markTaskStat(task, Task.TaskStatEnum.WAIT, "onWait");
    }else if (task.stat === "skip") {
        self.markTaskStat(task, Task.TaskStatEnum.SKIP);
    } else if (task.stat === "ABORT") {
        self.db.delById(task.id, function (err, data) {
            if (!err) {
                self.emit("onAbort", [task]);
            } else {
                global.logger.error('[DB]--error', err);
            }
        });
    }
    if (callback) {
        callback.call();
    }

};

/**
 * 标注task结束后进行持久化，并发送task end事件
 * @param Task
 * @param stat  SUCCESS/MD5SUCCESS/MD5FAILED
 */
BucketPool.prototype.markTaskStat = function (task, stat, event = "end") {
    var self = this;
    task.changeStat(stat);
    self.db.updateStat(task, function (err, data) {
        if (!err) {
            self.emit(event, [task]);
        }
        else {
            global.logger.error("[upload] task finished,but update stat failed.", task.path, task.stat, err)
        }
    });
};
/**
 *
 * @param 移动文件
 * 先用fs.rename 如果失败再执行复制删除
 */
BucketPool.prototype.move = function (task) {
    var stat = fs.lstatSync(task.path);
    var isDirectory = stat.isDirectory();
    var sourceFile = task.path;
    var destPath;
    if(isDirectory){
        destPath = path.join(task.movePath, task.fileName);
        var data = fs.readdirSync(task.path)
    }else{
        destPath = path.join(task.movePath, task.fileName);
        var fatherPath = destPath.substring(0,destPath.lastIndexOf("\\")+1);
        if(!fs.existsSync(fatherPath)){
            fs.mkdirSync(fatherPath);
        } else {
            logger.info("fatherPath exist:", fatherPath);
        }
        fs.rename(sourceFile, destPath, function (err) {
            if(err){
                global.logger.info("fs rename err and try to use stream pipe then unlink",err)
                var readStream = fs.createReadStream(sourceFile);
                var writeStream = fs.createWriteStream(destPath);
                readStream.pipe(writeStream);
                readStream.on('end',function(){
                    fs.unlinkSync(sourceFile);
                })
            }
        });
    }
}
/**
 *
 * @param task 此task必须在taskmanager的runningMap中
 * 没有serviceHandle的task，比如size为0的，文件夹这种，一旦进入runningMap，
 * 无法被暂停或者取消
 */
BucketPool.prototype.pause = function (task) {
    var serviceHandle = this.serviceHandles.get(task.id);
    if (serviceHandle) {
        if (serviceHandle.pause) {  //如果有暂停函数，表示分段上传，调用暂停
            serviceHandle.pause();
        } else {
            task.stat = Task.TaskStatEnum.PAUSE;
            task.progress = "0%";
            serviceHandle.abort();    //非分段上传，直接abort,sdk没有暂停方法
        }
    }
}

/**
 *
 * @param task 此task必须在taskmanager的runningMap中
 */
BucketPool.prototype.abort = function (task) {
    var serviceHandle = this.serviceHandles.get(task.id);
    task.stat = Task.TaskStatEnum.ABORT;
    if (serviceHandle) {
        serviceHandle.abort();
    }
}

AWS.util.mixin(BucketPool, AWS.SequentialExecutor);
module.exports = BucketPool;