/**
 * Created by yk on 2017/1/5.
 */
/**
 * error code : 1 2 3 4 5 6 7 8 9 10 11 12
 * 1-3位，产品标志位，001-OBS Browser，002-OBS Console，003-DES Console，004-DES ImportServer,...
 * 4-6位，对应模块标志，如：001001-OBS Browser任务管理模块，001002-OBS Browser上传模块，001003-OBS Browser下载模块, 001004-OBS Browser桶管理模块...
 * 7位，错误严重等级，如：1-提示，2-一般，3-警告，4-严重，5-致命
 * 8-11，错误信息编号，如：001001400011-[致命]OBS Browser任务管理模块无法连接到DB
 * 12位，错误来源，1-程序内部，2-依赖接口，3-操作系统
 *
 * @param name
 * @returns {*}
 * @constructor
 */
var obsError = function (name,message,statusCode) {
    /**
     *   code 错误码编号，在整个OBS域中全局唯一，非空
     name 错误名称，尽量简短，不唯一，非空
     show true，在UI上显示，false，不在UI上显示，在日志中打印
     description 错误描述，用一定文本描述错误信息，能描述出导致该错误的原因，非空
     suggestion 操作建议，如果是面向产品用户，则给出简单的操作建议或帮助链接； 如果是面对debug用户，则给出更详细的建议，如：请检查数据库是否启动；请检查与XX接口通信是否正常。
     origin 原始Error信息，捕获到的Err对象
     extInfo 根据code解析
     product 产品名称
     module 产品模块
     rank 错误等级
     cause 错误来源
     *
     * 样例
     ERROR_OBJECT={
            code:"001001400011",
            name:"Connect DB Failed",
            show:false,
            description:"Cannot access DB, xxxxx",
            suggestion:"Check does DB process exist",
            origin:{},
            extInfo:{
                product:"OBS Browser",
                module:"Upload",
                rank:"5",
                cause:"1"
            }
        }
     *
     */
    if(name == "NetworkingError"&& message && message.indexOf('self signed certificate')>-1){
        name = "SSL";
    }
    if(name == "AccessDenied"&& message && message.indexOf('Policy expired')>-1){
        name = "expired";
    }
    var obs_err = {
        "407":{
            code:"",
            name:"407",
            description:global.langData.ERR_CODE_407
        },
        "500":{
            code:"",
            name:"500",
            description:global.langData.ERR_CODE_500
        },
        "501":{
            code:"",
            name:"501",
            description:global.langData.ERR_CODE_501
        },
        "502":{
            code:"",
            name:"502",
            description:global.langData.ERR_CODE_502
        },
        "503":{
            code:"",
            name:"503",
            description:global.langData.ERR_CODE_503
        },
        "AccessDenied":{
            code:"",
            name:"AccessDenied",
            description:global.langData.ERR_CODE_AccessDenied
        },
        "expired":{
            code:"",
            name:"expired",
            description:global.langData.obs_term_accesscodeAccessDenied_msg
        },
        "AccountProblem":{
            code:"",
            name:"AccountProblem",
            description:global.langData.ERR_CODE_AccountProblem
        },"SSL":{
            code:"",
            name:"SSL",
            description:global.langData.obs_term_certificateNotValid_msg
        }, "AmbiguousGrantByEmailAddress":{
            code:"",
            name:"AmbiguousGrantByEmailAddress",
            description:global.langData.ERR_CODE_AmbiguousGrantByEmailAddress
        },
        "BadDigest":{
            code:"",
            name:"BadDigest",
            description:global.langData.ERR_CODE_BadDigest
        },
        "BucketAlreadyExists":{
            code:"",
            name:"BucketAlreadyExists",
            description:global.langData.ERR_CODE_BucketAlreadyExists
        },
        "BucketNotEmpty":{
            code:"",
            name:"BucketNotEmpty",
            description:global.langData.ERR_CODE_BucketNotEmpty
        },
        "CredentialsNotSupported":{
            code:"",
            name:"CredentialsNotSupported",
            description:global.langData.ERR_CODE_CredentialsNotSupported
        },
        "EBADF":{
            code:"",
            name:"EBADF",
            description:global.langData.ERR_CODE_EBADF
        },
        "EBUSY":{
            code:"",
            name:"EBUSY",
            description:global.langData.ERR_CODE_EBUSY
        },
        "EISDIR":{
            code:"",
            name:"EISDIR",
            description:global.langData.ERR_CODE_EISDIR
        },
        "ENOENT4058":{
            code:"",
            name:"ENOENT4058",
            description:global.langData.ERR_CODE_ENOENT4058
        },
        "ENOSPC":{
            code:"",
            name:"ENOSPC",
            description:global.langData.ERR_CODE_ENOSPC
        },
        "EntityTooLarge":{
            code:"",
            name:"EntityTooLarge",
            description:global.langData.ERR_CODE_EntityTooLarge
        },
        "EntityTooSmall":{
            code:"",
            name:"EntityTooSmall",
            description:global.langData.ERR_CODE_EntityTooSmall
        },
        "EPERM":{
            code:"",
            name:"EPERM",
            description:global.langData.ERR_CODE_EPERM
        },
        "InArrearOrInsufficientBalance":{
            code:"",
            name:"InArrearOrInsufficientBalance",
            description:global.langData.ERR_CODE_InArrearOrInsufficientBalance
        },
        "IncompleteBody":{
            code:"",
            name:"IncompleteBody",
            description:global.langData.ERR_CODE_IncompleteBody
        },
        "IncorrectNumberOfFilesInPostRequest":{
            code:"",
            name:"IncorrectNumberOfFilesInPostRequest",
            description:global.langData.ERR_CODE_IncorrectNumberOfFilesInPostRequest
        },
        "InlineDataTooLarge":{
            code:"",
            name:"InlineDataTooLarge",
            description:global.langData.ERR_CODE_InlineDataTooLarge
        },
        "InternalError":{
            code:"",
            name:"InternalError",
            description:global.langData.ERR_CODE_InternalError
        },
        "InvalidAccessKeyId":{
            code:"",
            name:"InvalidAccessKeyId",
            description:global.langData.ERR_CODE_InvalidAccessKeyId
        },
        "InvalidArgument":{
            code:"",
            name:"InvalidArgument",
            description:global.langData.ERR_CODE_InvalidArgument
        },
        "InvalidDigest":{
            code:"",
            name:"InvalidDigest",
            description:global.langData.ERR_CODE_InvalidDigest
        },
        "InvalidLocationConstraint":{
            code:"",
            name:"InvalidLocationConstraint",
            description:global.langData.ERR_CODE_InvalidLocationConstraint
        },
        "InvalidPart":{
            code:"",
            name:"InvalidPart",
            description:global.langData.ERR_CODE_InvalidPart
        },
        "InvalidPartOrder":{
            code:"",
            name:"InvalidPartOrder",
            description:global.langData.ERR_CODE_InvalidPartOrder
        },
        "InvalidPayer":{
            code:"",
            name:"InvalidPayer",
            description:global.langData.ERR_CODE_InvalidPayer
        },
        "InvalidPolicyDocument":{
            code:"",
            name:"InvalidPolicyDocument",
            description:global.langData.ERR_CODE_InvalidPolicyDocument
        },
        "InvalidRange":{
            code:"",
            name:"InvalidRange",
            description:global.langData.ERR_CODE_InvalidRange
        },
        "InvalidRedirectLocation":{
            code:"",
            name:"InvalidRedirectLocation",
            description:global.langData.ERR_CODE_InvalidRedirectLocation
        },
        "InvalidRequest":{
            code:"",
            name:"InvalidRequest",
            description:global.langData.ERR_CODE_InvalidRequest
        },
        "InvalidTargetBucketForLogging":{
            code:"",
            name:"InvalidTargetBucketForLogging",
            description:global.langData.ERR_CODE_InvalidTargetBucketForLogging
        },
        "KeyTooLong":{
            code:"",
            name:"KeyTooLong",
            description:global.langData.ERR_CODE_KeyTooLong
        },
        "MalformedPOSTRequest":{
            code:"",
            name:"MalformedPOSTRequest",
            description:global.langData.ERR_CODE_MalformedPOSTRequest
        },
        "MalformedXML":{
            code:"",
            name:"MalformedXML",
            description:global.langData.ERR_CODE_MalformedXML
        },
        "MaxMessageLengthExceeded":{
            code:"",
            name:"MaxMessageLengthExceeded",
            description:global.langData.ERR_CODE_MaxMessageLengthExceeded
        },
        "MaxPostPreDataLengthExceededError":{
            code:"",
            name:"MaxPostPreDataLengthExceededError",
            description:global.langData.ERR_CODE_MaxPostPreDataLengthExceededError
        },
        "MetadataTooLarge":{
            code:"",
            name:"MetadataTooLarge",
            description:global.langData.ERR_CODE_MetadataTooLarge
        },
        "MethodNotAllowed":{
            code:"",
            name:"MethodNotAllowed",
            description:global.langData.ERR_CODE_MethodNotAllowed
        },
        "MissingContentLength":{
            code:"",
            name:"MissingContentLength",
            description:global.langData.ERR_CODE_MissingContentLength
        },
        "MissingRequestBodyError":{
            code:"",
            name:"MissingRequestBodyError",
            description:global.langData.ERR_CODE_MissingRequestBodyError
        },
        "MissingRequiredParameter":{
            code:"",
            name:"MissingRequiredParameter",
            description:global.langData.ERR_CODE_MissingRequiredParameter
        },
        "MissingSecurityHeader":{
            code:"",
            name:"MissingSecurityHeader",
            description:global.langData.ERR_CODE_MissingSecurityHeader
        },
        "NetworkingError":{
            code:"",
            name:"NetworkingError",
            description:global.langData.ERR_CODE_NetworkingError
        },
        "NetworkingError1":{
            code:"",
            name:"NetworkingError1",
            description:global.langData.ERR_CODE_NetworkingError1
        },
        "NoSuchBucket":{
            code:"",
            name:"NoSuchBucket",
            description:global.langData.ERR_CODE_NoSuchBucket
        },
        "NoSuchFile":{
            code:"",
            name:"NoSuchFile",
            description:global.langData.ERR_CODE_NoSuchFile
        },
        "NoSuchKey":{
            code:"",
            name:"NoSuchKey",
            description:global.langData.ERR_CODE_NoSuchKey
        },
        "NoSuchUpload":{
            code:"",
            name:"NoSuchUpload",
            description:global.langData.ERR_CODE_NoSuchUpload
        },
        "NoSuchVersion":{
            code:"",
            name:"NoSuchVersion",
            description:global.langData.ERR_CODE_NoSuchVersion
        },
        "NotImplemented":{
            code:"",
            name:"NotImplemented",
            description:global.langData.ERR_CODE_NotImplemented
        },
        "OperationAborted":{
            code:"",
            name:"OperationAborted",
            description:global.langData.ERR_CODE_OperationAborted
        },
        "PermanentRedirect":{
            code:"",
            name:"PermanentRedirect",
            description:global.langData.ERR_CODE_PermanentRedirect
        },
        "RegionNameNotMatch":{
            code:"",
            name:"RegionNameNotMatch",
            description:global.langData.ERR_CODE_RegionNameNotMatch
        },
        "RequestTimeout":{
            code:"",
            name:"RequestTimeout",
            description:global.langData.ERR_CODE_RequestTimeout
        },
        "RequestTimeTooSkewed":{
            code:"",
            name:"RequestTimeTooSkewed",
            description:global.langData.ERR_CODE_RequestTimeTooSkewed
        },
        "SignatureDoesNotMatch":{
            code:"",
            name:"SignatureDoesNotMatch",
            description:global.langData.ERR_CODE_SignatureDoesNotMatch
        },
        "SlowDown":{
            code:"",
            name:"SlowDown",
            description:global.langData.ERR_CODE_SlowDown
        },
        "someObjectDelErr":{
            code:"",
            name:"someObjectDelErr",
            description:global.langData.ERR_CODE_someObjectDelErr
        },
        "someObjectDownErr":{
            code:"",
            name:"someObjectDownErr",
            description:global.langData.ERR_CODE_someObjectDownErr
        },
        "TooManyBuckets":{
            code:"",
            name:"TooManyBuckets",
            description:global.langData.ERR_CODE_TooManyBuckets
        },
        "TooManyWrongSignature":{
            code:"",
            name:"TooManyWrongSignature",
            description:global.langData.ERR_CODE_TooManyWrongSignature
        },
        "undefinedError":{
            code:"",
            name:"undefinedError",
            description:global.langData.ERR_CODE_undefinedError
        },
        "UnknownEndpoint":{
            code:"",
            name:"UnknownEndpoint",
            description:global.langData.ERR_CODE_UnknownEndpoint
        },
        "XMLParserError":{
            code:"",
            name:"XMLParserError",
            description:global.langData.ERR_CODE_XMLParserError
        },
        "KeyTooLongError":{
            code:"",
            name:"KeyTooLongError",
            description:global.langData.ERR_CODE_KeyTooLongError
        },
        "ProxyError":{
            code:"",
            name:"ProxyError",
            description:global.langData.ERR_CODE_ProxyError
        },
        "AllAccessDisabled":{
            code:"",
            name:"AllAccessDisabled",
            description:global.langData.ERR_CODE_AllAccessDisabled
        },
        "InvalidTargetBucketForLoggingRegion":{
            code:"",
            name:"InvalidTargetBucketForLoggingRegion",
            description:global.langData.ERR_CODE_InvalidTargetBucketForLoggingRegion
        },
        "AuthorizationHeaderMalformed":{
            code:"",
            name:"AuthorizationHeaderMalformed",
            description:global.langData.ERR_CODE_AuthorizationHeaderMalformed
        },
        "IllegalLocationConstraintException":{
            code:"",
            name:"IllegalLocationConstraintException",
            description:global.langData.ERR_CODE_IllegalLocationConstraintException
        },
        "ENOENT":{
            code:"",
            name:"ENOENT",
            description:global.langData.ERR_CODE_ENOENT
        },
        "ServiceUnavailable":{
            code:"",
            name:"ServiceUnavailable",
            description:global.langData.ERR_CODE_ServiceUnavailable
        },
        "searchPrefixTooLong":{
            code:"",
            name:"searchPrefixTooLong",
            description:global.langData.ERR_CODE_searchPrefixTooLong
        },
        "RestoreAlreadyInProgress": {
            code:"",
            name:"RestoreAlreadyInProgress",
            description:global.langData.ERR_CODE_RestoreAlreadyInProgress
        },
        "InvalidObjectState": {
            code:"",
            name:"InvalidObjectState",
            description:global.langData.ERR_CODE_InvalidObjectState
        },
        "NotFound": {
            code:"",
            name:"NotFound",
            description:global.langData.ERR_CODE_NotFound
        },
        "Forbidden": {
            code:"",
            name:"Forbidden",
            description:global.langData.ERR_CODE_Forbidden
        },
        "obs_task_addLifecycleFail_prefixOverlap_msg": {
            code:"",
            name:"obs_task_addLifecycleFail_prefixOverlap_msg",
            description:global.langData.obs_task_addLifecycleFail_prefixOverlap_msg
        },
        "lifecycle_duplicate_ruleid": {
            code:"",
            name:"lifecycle_duplicate_ruleid",
            description:global.langData.ERR_CODE_lifecycle_duplicate_ruleid
        },
        "KMS.KMSInvalidStateException": {
            code:"",
            name:"KMS.KMSInvalidStateException",
            description:global.langData["KMS.KMSInvalidStateException"]
        },
        "FsNotSupport": {
            code:"",
            name:"FsNotSupport",
            description:global.langData.ERR_CODE_FsNotSupport
        }
    };
    var result = obs_err[name];
    if(!result&&statusCode){
        name = statusCode;
    }
    return obs_err[name];
}
module.exports = obsError;
