/**
 * Created on 5/20/2017.
 */
var async = require("async");
var TaskEnum = require("../common/TaskEnum.js");


function CopyMoveComponent(taskControlAction,taskManager,targetBucket,targetPath){
    this.copyMoveItems=[];
    for(let item of global.glbClipboard.contents){
        this.copyMoveItems.push(new CopyMoveItem(taskControlAction, taskManager, targetBucket, targetPath,item))
    }
}

function CopyMoveItem(taskControlAction, taskManager, targetBucket, targetPath,item) {
    this.taskControlAction = taskControlAction;
    this.taskManager = taskManager;
    this.targetBucket = targetBucket;
    this.targetPath = targetPath;
    this.sourceBucket = item.bucket;
    this.sourceKey = item.key;
    this.size = item.size;
    this.action = global.glbClipboard.action;
    this.objName = item.objName;
    this.isFolder = item.isFolder;
    this.sourceFsStatus = item.sourceFsStatus;
}

CopyMoveComponent.prototype.doPaste = function (callback) {
    let tasks = [];
    for (let item of this.copyMoveItems) {
        tasks.push(new Promise(function (resolve, reject) {
            item.doPaste(function (err) {
                if (err) {
                    reject(err);
                } else {
                    resolve();
                }
            })
        }))
    }
    Promise.all(tasks).then(function () {
        callback();
    }).catch((err) => {
        callback(err);
    });
}

/**
 * 执行粘贴逻辑
 * @param callBack
 */
CopyMoveItem.prototype.doPaste = function(callBack) {
    var self = this;
    var sliptStart = self.sourceKey.lastIndexOf(self.objName);
    function _errProcess(err, callback) {
        global.logger.error("Copy/Move listObject err.", err);
        callback(err);
    }

    function _actionProcess(data) {
        async.eachLimit(data.Contents, 20, function (item, callback) {
            //需要获取以当前folder名称为开头的字符串子串
            var targetKeyPart = item.Key.substr(sliptStart);
            self.taskControlAction.pasteObject(
                self.targetBucket,
                self.targetPath + targetKeyPart,
                self.sourceBucket,
                item.Key,
                item.Size,
                self.action,
                self.sourceFsStatus,
                TaskEnum.TASK_STAT.wait.value);
            callback();
        }, function (cb) {
            if (cb) {
                cb();
            }
        });

    }

    function _batchProcess(params, _errProcess, _actionProcess, callback) {
        if (self.taskManager.abortAll) {
            return;
        }
        s3.listObjects(params, function (err, data) {
            if (err) {
                // 处理err
                _errProcess(err, callback);
            } else if (data.Contents && data.Contents.length > 0) {
                // 添加到任务中去
                _actionProcess(data);
                if (data.IsTruncated) {
                    var nextParams = {
                        Bucket: params.Bucket,
                        Prefix: params.Prefix,
                        Marker: data.NextMarker,
                        MaxKeys: params.MaxKeys
                    };
                    _batchProcess(nextParams, _errProcess, _actionProcess, callback);
                } else {
                    callback();
                }
            } else {
                _errProcess({code: "NoSuchFile"}, callback);
            }
        });
    }

    if (self.isFolder) {
        // 如果是文件语义的桶并且是不跨桶的移动操作则直接调用rename接口，不用递归出里面的对象
        if(global.curFileInterface === "Enabled" && self.targetBucket === self.sourceBucket && self.action === "move"){
            global.logger.info("lingnan");
            this.taskControlAction.pasteObject(
                self.targetBucket,
                self.targetPath + self.objName,
                self.sourceBucket,
                self.sourceKey,
                0,
                self.action,
                self.sourceFsStatus,
                TaskEnum.TASK_STAT.wait.value);
            callBack();
        }else{
            //如果是文件夹，则遍历文件夹下面的所有对象
            var params = {Bucket: self.sourceBucket, Prefix: self.sourceKey, MaxKeys: 100};
            _batchProcess(params, _errProcess, _actionProcess, function (err) {
                global.logger.info("Copy/Move task added successfully.");
                callBack(err);
            });
        }
    } else {
        //如果是对象，则直接添加任务
        this.taskControlAction.pasteObject(
            self.targetBucket,
            self.targetPath + self.objName,
            self.sourceBucket,
            self.sourceKey,
            self.size,
            self.action,
            self.sourceFsStatus,
            TaskEnum.TASK_STAT.wait.value);
        callBack();
    }
}


module.exports = CopyMoveComponent;