/**
 * 一些工具类函数
 * @type {{}}
 * @private
 */

function InvokeToolModel(funcObj, args, interval) {
    this.funcObj = funcObj;          //函数实际对象
    this.args = args;                 //函数参数
    this.lastInvokeTime = 0;  //函数上次执行时间
    this.interval = interval > 0 ? interval : invokeTool.DefaultInterval;


    this.invokeHandle = null;
}


InvokeToolModel.prototype.invokeWithInterval = function () {
    var currentTime = (new Date()).getTime();
    //如果大于间隔时间，则立即调用方法对象
    if ((currentTime - this.lastInvokeTime) > this.interval) {
        if (this.invokeHandle) {
            clearTimeout(this.invokeHandle);
            this.invokeHandle = null;
        }
        this.lastInvokeTime = currentTime;
        return this.funcObj(this.args);
    }
    else {
        //这个分支的原因是假如有n次函数调用，当n-1和n次的调用间隔小于interval时，可能导致函数最终少调用了一次，引起界面状态不一致
        if (!this.invokeHandle) {
            this.invokeHandle = setTimeout(this.invokeWithInterval.bind(this), this.interval + 1)
        }
        return undefined;
    }
}

InvokeToolModel.prototype.setAttri = function (funcObj, args, interval) {
    this.funcObj = funcObj;          //函数实际对象
    this.args = args;                 //函数参数
    this.interval = interval ? interval : invokeTool.DefaultInterval;
}


var invokeTool = {
    //保存函数操作和其上次运行时间,当字典用
    // key:函数名吧，value为:invokeToolModel
    _invokeToolModels: {},

    //默认执行间隔时间(大概2秒左右)
    DefaultInterval: 2003,

    /**
     * 有些函数不可运行得太过频繁,必须控制其再最短interval以上运行
     * @param operaName  操作名称 ( 最好取OperaNamesEnum中的值)
     * @param interval  最小间隔运行时间 (可选参数)
     * @returns {boolean}
     */
    invokeWithInterval: function (funcObj, args, interval, operaName) {
        var self = this;
        if (!operaName) {
            operaName = funcObj.name;
        }
        if (!self._invokeToolModels[operaName]) {
            self._invokeToolModels[operaName] = new InvokeToolModel(funcObj, args, interval)
        } else {
            self._invokeToolModels[operaName].setAttri(funcObj, args, interval);
        }
        self._invokeToolModels[operaName].invokeWithInterval();
    },
}

/**
 * task的id生成
 * @returns {string}
 * @private
 */
function guid() {
    var S4 = function () {
        return Math.floor(
            Math.random() * 0X10000 /*65536*/
        ).toString(16)
    };
    return S4() + S4() + "-" + S4() + "-" + S4() + "-" + S4() + "-" + S4() + S4() + S4();
}


// Array Remove - By John Resig (MIT Licensed)
var removeArrayItem = function (arrays, from, to) {
    from=parseInt(from);
    var rest = arrays.slice((to || from) + 1 || arrays.length);
    arrays.length = from < 0 ? arrays.length + from : from;
    return arrays.push.apply(arrays, rest);
};

module.exports.invokeTool = invokeTool;
module.exports.guid = guid;
module.exports.removeArrayItem = removeArrayItem;

