

var util = require("util"),EventEmitter =require("events").EventEmitter;
//添加事件监听机制
util.inherits(BucketAttributeAction,EventEmitter);
var logger;
function BucketAttributeAction(s3){
    this.s3 = s3;
}

util.inherits(BucketAttributeAction,EventEmitter);
var CommonUtils = require("../common/CommonUtils.js");


/**
 * 获取桶区域信息
 */
BucketAttributeAction.prototype.getBucketLocation=function (bucketName, callback) {
    var params = {
        Bucket: bucketName
    };
    s3.getBucketLocation(params, function (err, data) {
        if(callback){
            callback(err, data, bucketName);
        }
    });
};


/**
 获取桶存量信息
 */
BucketAttributeAction.prototype.getBucketStorageInfo=function(bucketName, callback) {
    var params = {
        Bucket: bucketName
    };
    global.logger.info('getBucketStorageInfo params:',params);
    s3.getBucketStorageInfo(params, function (err, data) {
        if(err&&err.statusCode==403&&err.code=='SignatureDoesNotMatch'){
            if(callback){
                callback(null, {});
            }
        }else{
            if(callback){
                callback(err, data);
            }
        }
    });
};
BucketAttributeAction.prototype.headBucket=function(bucketName, callback) {
    var params = {
        Bucket: bucketName
    };
    s3.headBucket(params, function (err, data) {
        if(data) {
            data.StorageType = CommonUtils.translateStorageClass(data.StorageType);
        }
        if(err&&err.statusCode==403&&err.code=='SignatureDoesNotMatch'){
            if(callback){
                callback(null, {});
            }
        }else{
            if(callback){
                callback(err, data, bucketName);
            }
        }
    });
};
//绑定域名
BucketAttributeAction.prototype.putBucketDomain=function(bucketName,DomainName,callback) {
    var params = {
        Bucket: bucketName,
        DomainName:DomainName
    };
    s3.putBucketDomain(params, function (err, data) {
        if(callback){
            callback(err, data);
        }
    });
};

/**
 获取桶配额信息
 */
BucketAttributeAction.prototype.getBucketQuota=function(bucketName, callback) {
    var params = {
        Bucket: bucketName
    };
    global.logger.info('getBucketQuota params:',params);
    s3.getBucketQuota(params, function (err, data) {
        if(callback){
            callback(err, data);
        }
    });
};

/*
 获取桶的Acl
 */
BucketAttributeAction.prototype.getBucketAcl=function (bucketName, callback) {
    var params = {
        Bucket: bucketName
    };
    global.logger.info('getBucketAcl params:',params);
    s3.getBucketAcl(params, function (err, data) {
        if(callback){
            callback(err, data);
        }
    });
};

/*
 设置桶的Acl
 */
BucketAttributeAction.prototype.putBucketAcl=function(callback, bucketName, acl, accessControlPolicy) {

    var params = {
        Bucket: bucketName
    };
    //header里的x-amz-acl字符串
    if (acl) {
        params.ACL = acl;
    }

    //xml形式的body；与header二者取其一？
    if (accessControlPolicy) {
        params.AccessControlPolicy = accessControlPolicy;
    }
    global.logger.info('putBucketAcl params:',params);
    s3.putBucketAcl(params, function (err, data) {
        if (callback) {
            callback(err, data);
        }
    });

};

/*
获取桶策略
*/
BucketAttributeAction.prototype.getBucketPolicy=function(bucket,callback){
    var params = {
        Bucket: bucket
    };
    global.logger.info('getBucketPolicy params:',params);
    s3.getBucketPolicy(params, function (err, data){
        if(callback)
            callback(err,data);
    });
};

/*
 设置桶策略
 */
BucketAttributeAction.prototype.putBucketPolicy=function(bucket, policy, callback){
    var params = {
        Bucket: bucket,
        Policy: policy
    };
    global.logger.info('putBucketPolicy params:',params);
    s3.putBucketPolicy(params, function (err, data){
        if(callback){
            callback(err,data);
        }
    });
};

/**
 * 获取桶存储策略
 */
BucketAttributeAction.prototype.storagePolicy=function (bucketName, callback) {
    var params = {
        Bucket: bucketName
    };
    s3.storagePolicy(params, function (err, data) {
        if(callback){
            callback(err, data, bucketName);
        }
    });
};

/**
 * 修改存儲類型
 */
BucketAttributeAction.prototype.putBucketStoragePolicy=function (bucketName, storagePolicy, callback) {
    var params = {
        Bucket: bucketName,
        StoragePolicy : {
            DefaultStorageClass: storagePolicy
        }
    };
    s3.putBucketStoragePolicy(params, function (err, data) {
        if(callback){
            callback(err, data, bucketName);
        }
    });
};

/*
 删除桶策略
 */
BucketAttributeAction.prototype.delBucketPolicy=function(bucketName,callback){
    var params = {
        Bucket: bucketName
    };
    global.logger.info('delBucketPolicy params:',params);
    s3.deleteBucketPolicy(params, function (err,data) {
        if(callback){
            callback(err,data);
        }
    });
};

/*
获取桶的cors列表
 */
BucketAttributeAction.prototype.getBucketCors=function(bucketName,callback){
    var params = {
        Bucket: bucketName
    };
    global.logger.info('getBucketCors params:',params);
    s3.getBucketCors(params, function (err, data) {
        if (callback) {
            callback(err, data);
        }
    });
};

/*
 设置桶的cors列表
 */
BucketAttributeAction.prototype.putBucketCors=function(bucketName, obj, callback){
    var params = {
        Bucket: bucketName,
        CORSConfiguration: {
            CORSRules: obj
        }
    };
    global.logger.info('putBucketCors params:',params);
    s3.putBucketCors(params, function (err,data){
        if (callback) {
            callback(err, data);
        }
    });
};

/*
 删除桶的cors列表
 */
BucketAttributeAction.prototype.deleteBucketCors=function(bucketName,callback){
    var params = {
        Bucket: bucketName
    };
    global.logger.info('deleteBucketCors params:',params);
    s3.deleteBucketCors(params, function (err,data) {
        if (callback) {
            callback(err, data);
        }
    });
};

/*
获得桶创建时间
*/
BucketAttributeAction.prototype.getBucketCreateTime=function(bucketName, callback) {
    s3.listBuckets(function (err, data) {
        if(callback){
            callback(err, data);
        }
    });
};

/*
 获取桶AclLog
 */
BucketAttributeAction.prototype.getBucketAclLog=function(bucketName,callback){
    var params = {
        Bucket: bucketName
    };
    s3.getBucketAcl(params, function (err, data) {
        if (callback) {
            callback(err, data);
        }
    });
};

/*
 设置桶的Logging
 */
BucketAttributeAction.prototype.putBucketLogging=function (bucketName, log, path,callback,isDel) {
    this.getBucketAclLog(bucketName,function(err,data){
        if(err){
            if(callback){
                callback(err);
            }
        }else{
            var params;
            if(!isDel){
                params = {
                    Bucket: bucketName,
                    BucketLoggingStatus: {
                        LoggingEnabled: {
                            TargetBucket: log,
                            TargetPrefix: path,
                            TargetGrants: [{
                                Grantee: {
                                    Type: "CanonicalUser",
                                    ID: data.Owner.ID,
                                    DisplayName: data.Owner.DisplayName
                                },
                                Permission: "FULL_CONTROL"
                            }]
                        }
                    }
                };
            }else{
                params = {Bucket: bucketName,BucketLoggingStatus:{}}
            }
            global.logger.info('putBucketLogging params:',params);
            s3.putBucketLogging(params, function (err,data) {
                if(callback){
                    callback(err,data);
                }
            });
        }
    })
};

/*
日志管理设置
*/
BucketAttributeAction.prototype.getBucketLogging=function(bucket,callback){
    var params = {
        Bucket:bucket
    };
    global.logger.info('getBucketLogging params:',params);
    s3.getBucketLogging(params,function(err,data){
        if (callback){
            callback(err,data);
        }
    });
};

BucketAttributeAction.prototype.replaceSpecialCharacterLog=function(str,languageJson,callback){
    var parrent = /[<>?:*\\|\"]/g;
    var tem = '0';
    if (parrent.test(str)) {
        tem = str.replace(parrent,"");
        if(callback){
            callback(languageJson.inputVerificationLogMsg,tem);
        }
    }else{
        if(callback){
            callback('',tem);
        }
    }
};

/*
 存放桶生命周期
*/
BucketAttributeAction.prototype.putBucketLifecycle=function(bucketName,arr,callback,language){
    if(arr.length>100){
        callback("error");
        return false;
    }
    var rules = [];
    var rule = {};
    for (var i = 0; i < arr.length; i++) {
        if(!arr[i].rulename){
            callback("rulename is not null");
            break;
            return false;
        }
        rule.ID = arr[i].rulename;
        rule.Prefix = arr[i].rulepath;
        rule.Status = arr[i].status ==="true" ? "Enabled" : "Disabled";
        rule.Expiration = {};
        rule.Transition = [];
        rule.NoncurrentVersionTransition = [];
        rule.NoncurrentVersionExpiration = {};
        var day,day1,day2,day3,day4,day5;
        if (arr[i].days.indexOf(language.day) > -1 && arr[i].days1.indexOf(language.day) > -1 ) {
            day = arr[i].days.replace(/[^0-9]/ig,"");
            day1 = arr[i].days1.replace(/[^0-9]/ig,"");
            day = parseInt(day);
            day1 = parseInt(day1);
            rule.Transition.push({
                "Days":day,
                "StorageClass":arr[i].daysStorage
            });
            rule.Transition.push({
                "Days":day1,
                "StorageClass":arr[i].days1Storage
            });
        }else if(arr[i].days.indexOf(language.day) > -1){
            day = arr[i].days.replace(/[^0-9]/ig,"");
            day = parseInt(day);
            rule.Transition.push({
                "Days":day,
                "StorageClass":arr[i].daysStorage
            });
        }else if(arr[i].days1.indexOf(language.day) > -1){
            day1 = arr[i].days1.replace(/[^0-9]/ig,"");
            day1 = parseInt(day1);
            rule.Transition.push({
                "Days":day1,
                "StorageClass":arr[i].days1Storage
            });
        }else{

        }
        if (arr[i].days2.indexOf(language.day) > -1) {
            day2 = arr[i].days2.replace(/[^0-9]/ig,"");
            day2 = parseInt(day2);
            rule.Expiration.Days = day2;
        }else{
            rule.Expiration = null;
        }
        if(arr[i].days3.indexOf(language.day) > -1 && arr[i].days4.indexOf(language.day) > -1){
            day3 = arr[i].days3.replace(/[^0-9]/ig,"");
            day4 = arr[i].days4.replace(/[^0-9]/ig,"");
            day3 = parseInt(day3);
            day4 = parseInt(day4);
            rule.NoncurrentVersionTransition.push({
                "NoncurrentDays":day3,
                "StorageClass":arr[i].days3Storage
            });
            rule.NoncurrentVersionTransition.push({
                "NoncurrentDays":day4,
                "StorageClass":arr[i].days4Storage
            });
        }else if(arr[i].days3.indexOf(language.day) > -1){
            day3 = arr[i].days3.replace(/[^0-9]/ig,"");
            day3 = parseInt(day3);
            rule.NoncurrentVersionTransition.push({
                "NoncurrentDays":day3,
                "StorageClass":arr[i].days3Storage
            });
        }else if(arr[i].days4.indexOf(language.day) > -1){
            day4 = arr[i].days4.replace(/[^0-9]/ig,"");
            day4 = parseInt(day4);
            rule.NoncurrentVersionTransition.push({
                "NoncurrentDays":day4,
                "StorageClass":arr[i].days4Storage
            });
        }
        if(arr[i].days5.indexOf(language.day) > -1){
            day5 = arr[i].days5.replace(/[^0-9]/ig,"");
            day5 = parseInt(day5);
            rule.NoncurrentVersionExpiration.NoncurrentDays = day5;
        }else{
            rule.NoncurrentVersionExpiration = null;
        }
        global.logger.info("参数",rule)
        rules.push(rule);
        rule = {};
    }
    var params = {
        Bucket:bucketName,
        LifecycleConfiguration: {
            Rules: rules
        }
    };
    global.logger.info('putBucketLifecycle params:',params,rules);
    s3.putBucketLifecycle(params,function(err,data){
        if (callback){
            callback(err,data);
        }
    });
};

/*
 设置桶lifecycle
*/
BucketAttributeAction.prototype.getBucketLifecycle = function(bucket,callback){
    var params = {
        Bucket:bucket
    };
    global.logger.info('getBucketLifecycle params:',params);
    s3.getBucketLifecycle(params,function(err,data){
        if (callback){
            callback(err,data);
        }
    });
};

/*
 删除桶lifecycle
 */
BucketAttributeAction.prototype.deleteBucketLifecycle=function(bucketName,callback){
    var params = {
        Bucket: bucketName
    };
    global.logger.info('deleteBucketLifecycle params:',params);
    s3.deleteBucketLifecycle(params, function (err,data) {
        if (callback){
            callback(err,data);
        }
    });
};

/**
 * 校验允许的来源
 * @param str
 * @param languageJson
 * @returns {boolean}
 */
BucketAttributeAction.prototype.checkAllowedOrigin=function(str){
    var arr =  str.split("\n");
    var result = false;
    for (var i=0; i < arr.length;i++){   //每行最多出现一个全角/半角的*号
        if(arr[i].split("*").length>2||arr[i].split("＊").length>2||(arr[i].split("*").length>1&&arr[i].split("＊").length>1)){
            result = true;
        }
    }
    return result;
};



/**
 * 校验允许的头域和补充的头域
 * @param str
 * @returns {boolean}
 */
BucketAttributeAction.prototype.checkAllowedAndExposeHeader = function(str,reg){
    var arr =  str.split("\n");
    for (var i=0; i < arr.length;i++){   //每行最多出现一个*号
        if (reg.test(arr[i])) {
            return true;
        }
        //Allowed Header, 判断每行是否存在两个*
        if(arr[i].split("*").length>2){
            return true;
        }
    }

};

module.exports = BucketAttributeAction;
