var CommonUtils = require("../common/CommonUtils.js");
var obsError = require("../common/ObsError.js");
var Task = require("../TaskModel").Task;
var AWS = require("hws-sdk/lib/core");
var EventEmitter = require("events").EventEmitter;
var async = require('async');

/**
 *  管理copy，move，rename等相关事宜
 *
 * @param service  sdk实现接口
 * @param task   任务,根据任务类型执行
 * @constructor
 */
function ManagedCopy(service, task) {
    EventEmitter.call(this);
    this.service = service;
    this.task = task;

    this.partCopy = undefined;

    let config = AWS.util.copy(this.service.config);
    this.service = new this.service.constructor.__super__(config);
}

ManagedCopy.prototype.__proto__ = EventEmitter.prototype;

ManagedCopy.prototype.abort = function () {
    if (this.partCopy) {
        this.partCopy.abort();
    }
}

ManagedCopy.prototype.pause = function () {
    if (this.partCopy) {
        this.partCopy.pause();
    }
}


ManagedCopy.prototype.process = function () {
    let self = this;
    if(self.task.fsStatus === "Enabled" && self.task.type != "copy"){
        // 如果是移动操作，需要先删除掉源文件，否则会报错(statusCode:409)
        if(self.task.type === "move"){
            // 移动(不跨桶)和重命名调用posix的rename接口
            if(self.task.bucket === self.task.sourceBucket){
                let headParams = {
                    Bucket: self.task.bucket,
                    Key: self.task.key
                };
                let promise = self.recursiveDeleteFolder(self.task.bucket, self.task.key, "", 0 , function () {});
                return promise.then(function () {
                    return self._renameObject();
                });

            }else {
                // 如果是跨桶操作，需要先删掉目标文件(夹)，再copy，最后再删除源文件(夹)
                let promise = self.recursiveDeleteFolder(self.task.bucket, self.task.key, "", 0 , function () {});
                logger.info("[move ]start task", self.task.type, self.task.key, self.task.size);
                return promise.then(() => {
                        return self._getObjectAcl();
                }).then(() => {
                        return self._headObject();
                }).then((headData) => {
                    return self._copyProcess(headData);
                }).then(() => {
                        return self._getObjectAcl();
                }).then((data) => {
                        return self._putObjectAcl(data);
                }).then(() => {
                    if(self.task.sourceFsStatus == "Enabled"){
                        return self.recursiveDeleteFolder(self.task.sourceBucket, self.task.sourceKey);
                    }else {
                        return self._deleteSource();
                    }
                });
            }
        }else {
            // 文件语义的桶重命名
            return self._renameObject();
        }
    }else{
        //let promise = self.recursiveDeleteFolder(self.task.bucket, self.task.key, "", 0 , function () {});
        logger.info("[move ]start task", self.task.type, self.task.key, self.task.size);
        let promise = self._getObjectAcl();
        return promise.then(() => {
                return self._headObject();
        }).then((headData) => {
                return self._copyProcess(headData);
        }).then(() => {
                return self._getObjectAcl();
        }).then((data) => {
                return self._putObjectAcl(data);
        }).then(() => {
            if(self.task.sourceFsStatus == "Enabled"){
                return self.recursiveDeleteFolder(self.task.sourceBucket, self.task.sourceKey);
            }else {
                return self._deleteSource();
            }
        });
    }
}

/*
 * 删除文件语义文件夹
 * bucket 桶名
 * parentPath 路径以"/"结尾
 * prefix 路径以"/"结尾
 */
ManagedCopy.prototype.recursiveDeleteFolder = function (bucket, prefix,marker,num,cb) {
    var self = this;
    if(!num){
        num = 0;
    }
    return new Promise((resolve, reject) => {
        var headParams = {Bucket: bucket, Key: prefix};
        if (self.task.type !== "move" && self.task.type !== "rename") {
            return resolve();
        }
        self.service.headObject(headParams, function (err, data) {
            if(err && err.statusCode === 404){
                // 不存在对象，不用执行删除
                return resolve();
            }
            // 如果是文件
            if(prefix.lastIndexOf("/") != prefix.length - 1){
                var delObjParams = {Bucket: bucket, Key: prefix};
                self.service.deleteObject(delObjParams, function (err) {
                    if (err) {
                        global.logger.error("[s3] move delObj", err);
                        return reject();
                    }
                    return resolve();
                });
            }else {
                var params = {Bucket: bucket, Prefix: prefix, Delimiter: "", Marker: marker, MaxKeys: 100};
                self.service.listObjects(params, function (err, data) {
                    if (err) {
                        if(num<4){
                            num++;
                            setTimeout(function () {
                                self.recursiveDeleteFolder(bucket, prefix, marker,num,cb);
                                global.logger.error('this is deletePath function error = ',num);
                            },num*60000);
                        }else{
                            if(cb){
                                cb(err);
                            }
                        }
                        global.logger.error(err);
                        return reject(err);
                    }
                    else{
                        num = 0;
                        if(data.Contents.length === 0){
                            var delObjParams = {Bucket: bucket, Key: prefix};
                            self.service.deleteObject(delObjParams, function (err) {
                                if (err) {
                                    global.logger.error("[s3] move delObj", err);
                                    return reject(err);
                                }
                                return resolve();
                            });
                            cb();
                        }
                        if (data.IsTruncated){
                            async.mapSeries(data.Contents, function(object, callback) {
                                setTimeout(function() {
                                    var key = object.Key;
                                    if(key.lastIndexOf("/") != key.length - 1){
                                        var delObjParams = {Bucket: bucket, Key: key};
                                        self.service.headObject(delObjParams, function (err, data) {
                                            if (err) {
                                                global.logger.error("[s3] move IsTruncated headObj", err);
                                                return reject();
                                            }
                                            self.service.deleteObject(delObjParams, function (err) {
                                                if (err) {
                                                    global.logger.error("[s3] move delObj", err);
                                                    callback(err);
                                                }
                                            });
                                        });
                                    }else {
                                        self.recursiveEmptyFolder(bucket, key, "", num, cb);
                                    }
                                    callback();
                                }, 5);
                            }, function(err,results) {
                                if(err){
                                    global.logger.error(err);
                                    return reject(err);
                                }
                                return resolve(results);
                            });
                            self.recursiveEmptyFolder(bucket, prefix, data.NextMarker,num,cb);
                        }else{
                            async.mapSeries(data.Contents, function(object, callback) {
                                setTimeout(function() {
                                    var key = object.Key;
                                    var delObjParams = {Bucket: bucket, Key: key};
                                    if(key.lastIndexOf("/") != key.length - 1){
                                        self.service.headObject(delObjParams, function (err, data) {
                                            if (err) {
                                                global.logger.error("[s3] move IsNotTruncated headObj", err);
                                                return reject();
                                            }
                                            self.service.deleteObject(delObjParams, function (err) {
                                                if (err) {
                                                    global.logger.error("[s3] move delObj", err);
                                                    callback(err);
                                                }
                                            });
                                        });
                                    }else {
                                        self.recursiveEmptyFolder(bucket, key, "", num, cb);
                                    }
                                    callback();
                                }, 5);
                            }, function(err,results) {
                                if(err){
                                    global.logger.error(err);
                                    return reject(err);
                                }
                                return resolve(results);
                            });
                        }
                    }
                });
            }
        });
    });

};

/*
 * 删除文件语义源文件/夹
 * bucket 桶名
 * parentPath 路径以"/"结尾
 * prefix 路径以"/"结尾
 */
ManagedCopy.prototype.recursiveDeleteSource = function (bucket, prefix,marker,num,cb) {
    var self = this;
    if(!num){
        num = 0;
    }
    return new Promise((resolve, reject) => {
        var headKeyParams = {Bucket: self.task.bucket, Key: self.task.key};
        var headParams = {Bucket: bucket, Key: prefix};
        if (self.task.type !== "move" && self.task.type !== "rename") {
            return resolve();
        }
        self.service.headObject(headKeyParams, function (err, data) {
            if (err) {
                global.logger.error("[s3] move recursiveDeleteSource headObj", err);
                self.task.stat = "FAILED";
                self.task.errMessage = CommonUtils.format(global.langData.obs_task_pasteObjectSuccess_getTargetObjectInfoFail_msg, obsError(err.code).description);
                return reject();
            }
            // 如果是文件
            if(prefix.lastIndexOf("/") != prefix.length - 1){
                var delObjParams = {Bucket: bucket, Key: prefix};
                self.service.deleteObject(delObjParams, function (err) {
                    if (err) {
                        global.logger.error("[s3] move delObj", err);
                        return reject();
                    }
                    return resolve();
                });
            }else {
                var params = {Bucket: bucket, Prefix: prefix, Delimiter: "", Marker: marker, MaxKeys: 100};
                self.service.listObjects(params, function (err, data) {
                    if (err) {
                        if(num<4){
                            num++;
                            setTimeout(function () {
                                self.recursiveDeleteFolder(bucket, prefix, marker,num,cb);
                                global.logger.error('this is deletePath function error = ',num);
                            },num*60000);
                        }else{
                            if(cb){
                                cb(err);
                            }
                        }
                        global.logger.error(err);
                        return reject(err);
                    }
                    else{
                        num = 0;
                        if(data.Contents.length <= 1){
                            var delObjParams = {Bucket: bucket, Key: prefix};
                            self.service.deleteObject(delObjParams, function (err) {
                                if (err) {
                                    global.logger.error("[s3] move delObj", err);
                                    return reject(err);
                                }
                                return resolve();
                            });
                            cb();
                        }
                        if (data.IsTruncated){
                            async.mapSeries(data.Contents, function(object, callback) {
                                setTimeout(function() {
                                    var key = object.Key;
                                    if(key.lastIndexOf("/") != key.length - 1){
                                        var delObjParams = {Bucket: bucket, Key: key};
                                        self.service.deleteObject(delObjParams, function (err) {
                                            if (err) {
                                                global.logger.error("[s3] move delObj", err);
                                                callback(err);
                                            }
                                            callback();
                                        });
                                    }else {
                                        self.recursiveEmptyFolder(bucket, key, "", num, cb);
                                    }
                                    callback();
                                }, 5);
                            }, function(err,results) {
                                if(err){
                                    global.logger.error(err);
                                    return reject(err);
                                }
                                return resolve(results);
                            });
                            self.recursiveEmptyFolder(bucket, prefix, data.NextMarker,num,cb);
                        }else{
                            async.mapSeries(data.Contents, function(object, callback) {
                                setTimeout(function() {
                                    var key = object.Key;
                                    var delObjParams = {Bucket: bucket, Key: key};
                                    if(key.lastIndexOf("/") != key.length - 1){
                                        self.service.deleteObject(delObjParams, function (err) {
                                            if (err) {
                                                global.logger.error("[s3] move delObj", err);
                                                callback(err);
                                            }
                                        });
                                    }else {
                                        self.recursiveEmptyFolder(bucket, key, "", num, cb);
                                    }
                                    callback();
                                }, 5);
                            }, function(err,results) {
                                if(err){
                                    global.logger.error(err);
                                    return reject(err);
                                }
                                return resolve(results);
                            });
                        }
                    }
                });
            }
        });
    });

};

/*
 * 删除空文件夹
 * bucket 桶名
 * parentPath 路径以"/"结尾
 * prefix 路径以"/"结尾
 */
ManagedCopy.prototype.recursiveEmptyFolder = function (bucket, prefix,marker,num,cb) {
    var self = this;
    if(!num){
        num = 0;
    }
    var params = {Bucket: bucket, Prefix: prefix, Delimiter: "", Marker: marker, MaxKeys: 100};
    self.service.listObjects(params, function (err, data) {
        if (err) {
            if(num<4){
                num++;
                setTimeout(function () {
                    self.recursiveEmptyFolder(bucket, prefix, marker,num,cb);
                    global.logger.error('this is deletePath function error = ',num);
                },num*60000);
            }else{
                if(cb){
                    cb(err);
                }
            }
            global.logger.error(err);
        }
        else{
            num = 0;
            var delObjParams = {Bucket: bucket, Key: prefix};
            if(data.Contents.length === 0){//文件夹已被删除
                self.service.deleteObject(delObjParams, function (err) {
                    if (err) {
                        global.logger.error("[s3] move delObj", err);
                    }
                });
                return false;
            }
            if(data.Contents.length === 1 && data.CommonPrefixes.length === 0){
                self.service.deleteObject(delObjParams, function (err) {
                    if (err) {
                        global.logger.error("[s3] move delObj", err);
                    }
                });
            }else {
                self.recursiveEmptyFolder(bucket, prefix, "",num,cb);
            }
        }
    });
};

ManagedCopy.prototype._deleteSource = function () {
    var self = this;
    var task = self.task;
    return new Promise((resolve, reject) => {
        if (task.type !== "move" && task.type !== "rename") {
            return resolve();
        }
        let headParams = {
            Bucket: task.bucket,
            Key: task.key
        };
        self.service.headObject(headParams, function (err, data) {
            if (err) {
                global.logger.error("[s3] move headObj", err);
                task.stat = "FAILED";
                task.errMessage = CommonUtils.format(global.langData.obs_task_pasteObjectSuccess_getTargetObjectInfoFail_msg, obsError(err.code).description);
                return reject();
            }
            var delObjParams = {
                Bucket: task.sourceBucket,
                Key: task.sourceKey
            };
            self.service.deleteObject(delObjParams, function (err) {
                if (err) {
                    global.logger.error("[s3] move delObj", err);
                    task.stat = "FAILED";
                    task.errMessage = CommonUtils.format(global.langData.obs_task_pasteObjectSuccess_delSourceobjectFail_msg, obsError(err.code).description);
                    return reject();
                }
                return resolve();
            });
        });
    });
}

/**
 * 先查看源文件是否存在,
 * @returns {Promise<any>}  reject(task)
 * @private
 */
ManagedCopy.prototype._headObject = function () {
    let self = this;
    let task = self.task;
    return new Promise((resolve, reject) => {
        let headParams = {
            Bucket: task.sourceBucket,
            Key: task.sourceKey
        };
        self.service.headObject(headParams, function (err, data) {
            if (err) {
                global.logger.error("[s3] copy obj", err, "params:", headParams);
                task.stat = Task.TaskStatEnum.FAILED;
                if (err.statusCode === 404) {
                    err.code = "NoSuchKey";
                }
                task.errMessage = CommonUtils.format(global.langData.obs_task_pasteObjectFail_msg, obsError(err.code).description);
                return reject();
            }
            resolve(data);
        });
    })
}

/**
 *  返回对象acl
 * @returns {Promise<any>}
 * @private
 */
ManagedCopy.prototype._getObjectAcl = function () {
    let self = this;
    let task = self.task;
    return new Promise((resolve, reject) => {
        let getAclParams = {
            Bucket: task.sourceBucket,
            Key: task.sourceKey
        }
        self.service.getObjectAcl(getAclParams, function (err, data) {
            if (err || (!data)) {
                global.logger.error("[s3] copy obj，get objACL", err, "params:", getAclParams);
                task.stat = "FAILED";
                task.errMessage = CommonUtils.format(global.langData.obs_task_pasteObjectSuccess_getSourceObjectAclFail_msg, obsError(err.code).description);
                return reject();
            }
            if (global.curOnwerId && global.curDisplayName) {
                data.Owner.ID = global.curOnwerId;
                data.Owner.DisplayName = global.curDisplayName;
            }
            resolve(data);
        });
    });
}

ManagedCopy.prototype._putObjectAcl = function (data) {
    let self = this;
    let task = self.task;
    return new Promise((resolve, reject) => {
        let putAclParams = {
            Bucket: task.bucket,
            Key: task.key,
            AccessControlPolicy: {
                Grants: data.Grants,
                Owner: data.Owner
            }
        };
        self.service.putObjectAcl(putAclParams, function (err, data) {
            if (err) {
                global.logger.error("[s3] copy obj，put objACL", err, "params:", putAclParams);
                self.task.stat = "FAILED";
                let totalMsg = CommonUtils.format(global.langData.obs_task_pasteObjectSuccess_setAclFail_msg, obsError(err.code).description)
                self.task.errMessage = totalMsg;
                return reject();
            }
            resolve();
        });
    });
}

ManagedCopy.prototype._renameObject = function (data) {
    let self = this;
    let task = self.task;
    self.emit("copyProgress", {loaded: 0, total: 100, part: 1});  //设置为99%
    return new Promise((resolve, reject) => {
        let params = {
            Bucket: task.bucket,
            objectName: task.sourceKey,
            dname: task.key
        }
        global.logger.error("[s3] _renameObject obj params", params);
        self.service.renameObject(params, function (err, data) {
            // 发生409冲突，重试
            if (err && err.statusCode === 409) {
                global.logger.error("[s3] rename obj 409 error:", err, "params:", params);
                // 重试
                setTimeout(function () {
                    self.service.renameObject(params, function (err, data) {
                        if (err) {
                            global.logger.error("[s3] rename obj error:", err, "params:", params);
                            task.stat = "FAILED";
                            task.errMessage = CommonUtils.format(global.langData.obs_task_pasteObjectFail_msg, obsError(err.code).description);
                            return reject();
                        }
                        return resolve();
                    });
                },1000);
            }
            return resolve();
        });
    });
}

ManagedCopy.prototype.copyEntirely = function (data) {
    let self = this;
    let task = self.task;
    self.emit("copyProgress", {loaded: 0, total: 100, part: 1});  //如果是整体拷贝，一律设置为99%
    return new Promise((resolve, reject) => {
        let source = "/" + task.sourceBucket + "/" + task.sourceKey;
        let params = {
            Bucket: task.bucket,
            Key: task.key,
            CopySource: encodeURIComponent(source)
        }
        if (data && data.SSEKMSKeyId && data.ServerSideEncryption) {
            params.SSEKMSKeyId = data.SSEKMSKeyId;
            params.ServerSideEncryption = data.ServerSideEncryption;
        }
        self.service.copyObject(params, function (err, data) {
            if (err) {
                global.logger.error("[s3] copyEntirely obj", err, "params:", params);
                task.stat = "FAILED";
                task.errMessage = CommonUtils.format(global.langData.obs_task_pasteObjectFail_msg, obsError(err.code).description);
                return reject();
            }
            resolve();
        });
    });
}

ManagedCopy.prototype._copyProcess = function (data) {
    let self = this;
    let task = self.task;

    if (task.size < 100 * 1024 * 1024) {  //100M
        return self.copyEntirely(data);
    } else {
        let params = {};
        if (data && data.SSEKMSKeyId && data.ServerSideEncryption) {
            params.SSEKMSKeyId = data.SSEKMSKeyId;
            params.ServerSideEncryption = data.ServerSideEncryption;
        }
        self.partCopy = new PartCopy(self.task, self.service, params);
        self.partCopy.on("copyProgress", function (evt) {
            self.emit("copyProgress", evt);
        });
        return self.partCopy.process();
    }
}


function PartCopy(task, service, uploadOption = {}) {
    EventEmitter.call(this);
    this.task = task;
    this.service = service;
    this.uploadOption = uploadOption;

    this.nextChunkStart = 0;  //下一段开始地方
    this.size = task.size;          //文件总大小

    this.partSize = task.partSize * 1024 * 1024;  //拷贝段段大小
    this.totalPartNumber = 0; //当前已经进行的分段数
    this.completeInfo = [];

    this.uploadIDRequest = undefined;  //获取上传ID
    this.parts = [];

    this.isStop = false;
    this.stopReasonErr = undefined;  //引起停止的原因
}

PartCopy.prototype.__proto__ = EventEmitter.prototype;


PartCopy.prototype.createUploadID = function () {
    let self = this;
    return new Promise((resolve, reject) => {
        if (!self.task.UploadId) {
            self.task.completeMap = {};
            let uploadOption = Object.assign({
                Bucket: self.task.bucket,
                Key: self.task.key
            }, self.uploadOption);
            logger.info("start to create copy part task upload id.", self.task.key, uploadOption, self.task.partSize);
            self.uploadIDRequest = self.service.createMultipartUpload(uploadOption, function (err, data) {
                self.uploadIDRequest = undefined;
                if (err) {
                    logger.error("create multipartupload ID error.", self.task, err);
                    return reject(err);
                }
                logger.info("create multipartupload ID finished.", self.task.key);
                self.task.UploadId = data.UploadId;
                return resolve();
            });
        } else {
            logger.info("[copy rename move] upload id is already exist.", self.task.key);
            return resolve();
        }
    });
}

PartCopy.prototype.createPartTask = function () {
    let self = this;
    if (self.nextChunkStart === self.size) {
        logger.info("copy part task has finished.", self.task.key);
        return Promise.resolve();
    }
    if (self.isStop) {
        return Promise.reject(self.stopReasonErr);
    }
    var promise = new Promise((resolve, reject) => {
        let start = self.nextChunkStart;
        let end = (self.nextChunkStart + self.partSize < self.size) ? self.nextChunkStart + self.partSize - 1 : self.size - 1
        self.nextChunkStart = end + 1;
        let partNumber = ++self.totalPartNumber;
        let totalParts = self.task.getTotalPartsNum();
        if (self.task.completeMap[partNumber]) {  //该段已经拷贝了
            self.completeInfo[partNumber] = {PartNumber: partNumber, ETag: self.task.completeMap[partNumber]};
            return resolve();
        }

        let partParams = {
            Bucket: self.task.bucket,
            Key: self.task.key,
            CopySource: encodeURIComponent("/" + self.task.sourceBucket + "/" + self.task.sourceKey),
            PartNumber: partNumber,
            UploadId: self.task.UploadId,
            CopySourceRange: "bytes=" + start + "-" + end
        };
        let req = self.service.uploadPartCopy(partParams);
        self.parts[partNumber] = req;
        req.send((err, data) => {
            delete self.parts[partNumber];
            if (self.isStop) {   //如果是停止造成的失败，则直接返回停止的原因
                logger.info("section stop!", self.task.key);
                return reject(self.stopReasonErr);
            }
            if (err) {
                logger.error("upload part copy failed.", self.task.key, err);
                return reject(err);
            }
            if ((!data) || !data.ETag) {
                var message = 'No access to ETag property on response.';
                err = AWS.util.error(new Error(message), {
                    code: 'ETagMissing',
                    retryable: false
                });
                return reject(err);
            }
            self.task.completeMap[partNumber] = data.ETag;
            self.completeInfo[partNumber] = {PartNumber: partNumber, ETag: data.ETag};
            self.emit("copyProgress", {loaded: self.completeInfo.length, total: totalParts + 1., part: partNumber});
            return resolve();
        });
    });
    return promise.then(() => {
        return self.createPartTask();
    })
}

/**
 * 如果已执行到这里，那么暂停和取消不生效
 * @returns {Promise<any>}
 */
PartCopy.prototype.completePartCopy = function () {
    let self = this;
    return new Promise((resolve, reject) => {
        let params = {
            Bucket: self.task.bucket,
            Key: self.task.key,
            MultipartUpload: {Parts: self.completeInfo},
            UploadId: self.task.UploadId
        };
        self.service.completeMultipartUpload(params, function (err) {
            if (err) {
                logger.error("complete partcopy failed. ", self.task, err);
                return reject(err);
            }
            self.task.completeMap = {};
            self.task.UploadId = undefined;
            logger.info("complete partcopy finished. ", self.task.key);
            resolve();
        });
    });
}

/**
 *  清理复制的段用，进到这里，最后还要返回reject
 * @param err
 * @returns {Promise<any>}
 */
PartCopy.prototype.cancelPartCopy = function (err) {
    let self = this;
    if (!err) {
        logger.warn("cancelPart copy error is undefined.", self.task.key)
        err = new Error("unknown error");
    }
    return new Promise((resolve, reject) => {
        logger.info("[partcopycancel] start cancel part copy.", self.task.key, self.task.type, err.code);
        self.cleanUp(err);
        if (err.code !== "RequestAbortedError") {  //如果不是用户主动取消，直接退出，不删除uploadid，以便继续执行
            return reject(err);
        }
        if (!self.task.UploadId) {
            logger.info("[partcopycancel]  no UploadId,just quit.", self.task.key);
            return reject(err);
        }
        let params = {
            Bucket: self.task.bucket,
            Key: self.task.key,
            UploadId: self.task.UploadId
        };
        self.service.abortMultipartUpload(params, (error) => {
            self.task.UploadId = undefined;
            if (error) {
                logger.error("[warn] abort MultipartUpload failed.", self.task.key, self.task.type, error);
            }
            logger.info("[partcopycancel] cancel part copy finished.", self.task.key, self.task.type);
            reject(err);
        });
    });
}

PartCopy.prototype.process = function () {
    let self = this;
    let promise = self.createUploadID();
    return promise.then(() => {
        logger.info("[partcopy] start create Part task............", self.task.key)
        let tasks = [];
        let querySize = self.task.queueSize;
        for (let index = 0; index < querySize; index++) {
            tasks.push(self.createPartTask())
        }
        return Promise.all(tasks);
    }).then(() => {
        return self.completePartCopy();
    }).catch((err) => {
        return self.cancelPartCopy(err);
    })
};

PartCopy.prototype.pause = function () {
    let self = this;
    self.cleanUp(AWS.util.error(new Error("Request paused by user"), {
        code: "RequestPausedError",
        retryable: false
    }));
}

PartCopy.prototype.abort = function () {
    let self = this;
    let reason = AWS.util.error(new Error('Request aborted by user'), {
        code: 'RequestAbortedError',
        retryable: false
    });
    self.stopReasonErr = reason;
    self.cleanUp(reason);
}

/**
 * sync
 */
PartCopy.prototype.cleanUp = function (err) {
    var self = this;
    if (self.isStop)
        return;
    self.isStop = true;
    self.stopReasonErr = err;
    try {
        self.parts.forEach(function (request) {
            request.abort();
        });
    }
    catch (err) {
        logger.error("abort request failed.", self.task.key, err);
    }
    self.parts = [];
}

module.exports = ManagedCopy;