/*
 任务控制接口
 */
var util = require("util"),EventEmitter =require("events").EventEmitter;
var async = require('async');
var os = require('os');
var fs = require('fs');
var Task = require("../TaskModel").Task
var LocalPathIterator = require('../common/LocalPathIterator.js');
var http = require('http');
var parseUrl = require('url');
var xml2js = require('xml2js');
var intervalTime = 10;//定时器间隔时间
//添加事件监听机制
util.inherits(TaskControlAction,EventEmitter);

var rootPath=process.execPath;
rootPath = rootPath.replace(/\\/g, "/");
rootPath = rootPath.substring(0,rootPath.lastIndexOf('/')+1);

var Const = require('../common/constants.js');
 var commonUtils = require('../common/CommonUtils.js');
function TaskControlAction(s3,taskManager){
    this.s3 = s3;
    this.taskManager = taskManager;
    this.fileQueue=[];
    this.iterators = [];
    this.iteratorsObject = true;
}

util.inherits(TaskControlAction,EventEmitter);

/*
 uuid生存算法
 */
function guid() {
    var S4 = function () {
        return Math.floor(
            Math.random() * 0X10000 /*65536*/
        ).toString(16)
    };
    return S4() + S4() + "-" + S4() + "-" + S4() + "-" + S4() + "-" + S4() + S4() + S4();
}

TaskControlAction.prototype.closeInterval=function(){
    clearInterval(this.time);
    this.taskManager.disableRunTask();
};

//删除一个对象
TaskControlAction.prototype.deleteObject = function (bucket, key ,size) {
    var self = this;
    if(!bucket || !key){
        return;
    }
    var uuid = guid();
    var task =
    {
        id: uuid,
        type: "delete",
        bucket: bucket,
        key: key,
        size: size,
        stat:"WAIT"
    };
    this.taskManager.push(task);
}
/*
 * 批量删除(文件夹)
 * bucket 桶名
 * parentPath 路径以"/"结尾
 * prefix 路径以"/"结尾
 */
TaskControlAction.prototype.deletePath = function (bucket, prefix,marker,num,cb) {
    var self = this;
    if(!self.iteratorsObject){
        return false;
    }
    if(!num){
        num = 0;
    }
    var params = {Bucket: bucket, Prefix: prefix, Delimiter: "", Marker: marker, MaxKeys: 100};
    if(!this.taskManager.abortAll){
        self.s3.listObjects(params, function (err, data) {
            if (err) {
                if(num<4){
                    num++;
                    setTimeout(function () {
                        self.deletePath(bucket, prefix, marker,num,cb);
                        global.logger.error('this is deletePath function error = ',num);
                    },num*60000);
                }else{
                    if(cb){
                        cb(err);
                    }
                }
                global.logger.error(err);
            }
            else{
                num = 0;
                if(data.Contents.length === 0){//文件夹已被删除
                    self.deleteObject(bucket, prefix ,0);
                    return false;
                }
                if (data.IsTruncated){
                    async.mapSeries(data.Contents, function(object, callback) {
                        setTimeout(function() {
                            var key = object.Key;
                            self.deleteObject(bucket, key ,object.Size);
                            if(self.taskManager.abortAll)
                                callback("abort by user");
                            else
                                callback();
                        }, 5);
                    }, function(err,results) {
                        if(err){
                            global.logger.error(err);
                        }
                    });
                    self.deletePath(bucket, prefix, data.NextMarker,num,cb);
                }else{
                    async.mapSeries(data.Contents, function(object, callback) {
                        setTimeout(function() {
                            var key = object.Key;
                            self.deleteObject(bucket, key ,object.Size);
                            if(self.taskManager.abortAll)
                                callback("abort by user");
                            else
                                callback();
                        }, 5);
                    }, function(err,results) {
                        if(err){
                            global.logger.error(err);
                        }
                    });
                }
            }
        });
    }
};

/*
 * 删除文件语义文件夹
 * bucket 桶名
 * parentPath 路径以"/"结尾
 * prefix 路径以"/"结尾
 */
TaskControlAction.prototype.recursiveDeleteFolder = function (bucket, prefix,marker,num,cb) {
    var self = this;
    if(!self.iteratorsObject){
        return false;
    }
    if(!num){
        num = 0;
    }

    var params = {Bucket: bucket, Prefix: prefix, Delimiter: "", Marker: marker, MaxKeys: 100};
    if(!this.taskManager.abortAll){
        self.s3.listObjects(params, function (err, data) {
            if (err) {
                if(num<4){
                    num++;
                    setTimeout(function () {
                        self.recursiveDeleteFolder(bucket, prefix, marker,num,cb);
                        global.logger.error('this is deletePath function error = ',num);
                    },num*60000);
                }else{
                    if(cb){
                        cb(err);
                    }
                }
                global.logger.error(err);
            }
            else{
                num = 0;
                if(data.Contents.length === 0){
                    self.deleteObject(bucket, prefix ,0);
                    return false;
                }
                if (data.IsTruncated){
                    async.mapSeries(data.Contents, function(object, callback) {
                        setTimeout(function() {
                            var key = object.Key;
                            if(key.lastIndexOf("/") != key.length - 1){
                                self.deleteObject(bucket, key ,object.Size);
                            }else {
                                self.recursiveEmptyFolder(bucket, key, "", num, cb);
                            }
                            if(self.taskManager.abortAll)
                                callback("abort by user");
                            else
                                callback();
                        }, 5);
                    }, function(err,results) {
                        if(err){
                            global.logger.error(err);
                        }
                    });
                    self.recursiveDeleteFolder(bucket, prefix, data.NextMarker,num,cb);
                }else{
                    async.mapSeries(data.Contents, function(object, callback) {
                        setTimeout(function() {
                            var key = object.Key;
                            if(key.lastIndexOf("/") != key.length - 1){
                                self.deleteObject(bucket, key ,object.Size);
                            }else {
                                self.recursiveEmptyFolder(bucket, key, "", num, cb);
                            }
                            if(self.taskManager.abortAll)
                                callback("abort by user");
                            else
                                callback();
                        }, 5);
                    }, function(err,results) {
                        if(err){
                            global.logger.error(err);
                        }
                    });
                }
            }
        });
    }
};

/*
 * 删除空文件夹
 * bucket 桶名
 * parentPath 路径以"/"结尾
 * prefix 路径以"/"结尾
 */
TaskControlAction.prototype.recursiveEmptyFolder = function (bucket, prefix,marker,num,cb) {
    var self = this;
    if(!self.iteratorsObject){
        return false;
    }
    if(!num){
        num = 0;
    }
    var params = {Bucket: bucket, Prefix: prefix, Delimiter: "", Marker: marker, MaxKeys: 100};
    if(!this.taskManager.abortAll){
        self.s3.listObjects(params, function (err, data) {
            if (err) {
                if(num<4){
                    num++;
                    setTimeout(function () {
                        self.recursiveEmptyFolder(bucket, prefix, marker,num,cb);
                        global.logger.error('this is deletePath function error = ',num);
                    },num*60000);
                }else{
                    if(cb){
                        cb(err);
                    }
                }
                global.logger.error(err);
            }
            else{
                num = 0;
                if(data.Contents.length === 0){//文件夹已被删除
                    self.deleteObject(bucket, prefix ,0);
                    return false;
                }
                if(data.Contents.length === 1 && data.CommonPrefixes.length === 0){
                    self.deleteObject(bucket, prefix ,0);
                }else {
                    self.recursiveEmptyFolder(bucket, prefix, "",num,cb);
                }
            }
        });
    }
};

/*
 上传文件
 */
TaskControlAction.prototype.uploadFile = function(bucket, key, filePath, storageType,movePath,fileName,compare, isPosixBucket) {
    var self = this;
    if(this.taskManager.abortAll) {
        return;
    }
    if (!(bucket && key && filePath)) {
        logger.info("upload file failed.not enough paramters: bucket,key, filepath", bucket, key, filePath);
    }
    filePath = filePath.replace(/[']/g, '\'');
    key = key.replace(/:/g, "$");
    global.logger.info("上传文件",fileName, !isPosixBucket);
    var ContentType = self.setContentTypes(fileName);
    fs.lstat(filePath, function (err, data) {
        var task = new Task(
            {
                type: Task.TaskTypeEnum.PUT,
                bucket: bucket,
                stat: Task.TaskStatEnum.WAIT,
                key: key,
                path: filePath,
                movePath:movePath,
                fileName:fileName,
                storagePolicy: storageType,
                uploadACL: 'bucket-owner-full-control',
                size: 0,
                progress:"0%",
                serverEncryption: global.serverEncryption && !isPosixBucket,
                ContentType:ContentType,
                compare:compare
            });
        if (err) {
            global.logger.error(err);
            var errCode = err.code;
            var errNo = err.errno;
            errNo = errNo.substring(1, errNo.length);
            errCode = errCode + errNo;
            task.stat = Task.TaskStatEnum.FAILED
            task.errMessage = errCode;
        } else {
            task.size = data.size;
            if (self.taskManager.pauseAll) {
                task.stat = 'PAUSE';
            }
        }
        self.taskManager.push(task);
    });

};

/*
 上传文件夹
 */
TaskControlAction.prototype.uploadPath = function(bucket ,prefix ,path, storageType,movePath,compare) {

    global.logger.info('[upload Path]',bucket ,prefix ,path, storageType);

    var start = path.substring(0, path.lastIndexOf("/", path.length - 2) + 1);
    var now = new Date();
    var self = this;
    var pathIterator = new LocalPathIterator({id:'itr-' + now.getTime(), bucket:bucket, prefix:prefix, path:start}, [path]);

    pathIterator.on('fileFound', function(itr,fileInfo){
        var realPath = itr.option.path.replace(/:/g, "$");
        //上传对象push到任务队列
        var file = fileInfo;

        var key = itr.option.prefix + file.path.substring(realPath.length);
        var fInfo = {
            bucket:itr.option.bucket,
            key:key,
            size:file.size,
            path:file.path,
            storageType:storageType,
            movePath:movePath,
            compare:compare,
            fileName:file.path.substring(realPath.length)
        };
        self.uploadFile(fInfo.bucket, fInfo.key, fInfo.path, fInfo.storageType,fInfo.movePath,fInfo.fileName,fInfo.compare);
    });

    pathIterator.on('floderFound',function(itr){
        var realPath = itr.option.path.replace(/:/g, "$");
        //上传对象push到任务队列
        var floder = itr.path;

        var key = itr.option.prefix + floder.substring(realPath.length) + "/";
        var fInfo = {
            bucket:itr.option.bucket,
            key:key,
            size:0,
            path:floder,
            storageType:storageType,
            movePath:movePath,
            compare:compare,
            fileName:floder.substring(realPath.length) + "/"
        };
        self.uploadFile(fInfo.bucket, fInfo.key, fInfo.path, fInfo.storageType,fInfo.movePath,fInfo.fileName,fInfo.compare);
    });

    pathIterator.on('finished',function(){
        global.logger.info("Iterator run finished.")
        for(var k in self.iterators){
            if(this.option.id == self.iterators[k].option.id){
                self.iterators[k]=null;
                delete  self.iterators[k];
                break;
            }
        }
    });

    pathIterator.start();

    self.iterators.push(pathIterator);
};
/*
 检查下载文件存放路径是否存在
 */
TaskControlAction.prototype.pathCheck = function(paths, callback) {
    var fs = require('fs');
    var path = require('path');
    paths = path.normalize(paths);
    fs.exists(paths, function (exist) {
        if (callback) {
            callback(exist);
        }
    })
};

/*
 下载文件
 bucket:bucketname
 key:key like 'a/b/c.txt'
 filepath:文件全路径like'd:/c.txt'
 */
TaskControlAction.prototype.downloadFile = function(bucket,key,filePath,size,etag) {
    var self = this;
    if (bucket && key && filePath) {
        var task = new Task({
            type: Task.TaskTypeEnum.GET,
            bucket: bucket,
            key: key,
            path: filePath,
            size: size,
            etag: etag,
            progress:"0%"
        });
        this.taskManager.push(task);
    }
};

TaskControlAction.prototype.setContentTypes = function(str) {
    const contentTypes = {
        "001":      "application/x-001",
        "301":      "application/x-301",
        "323":      "text/h323",
        "7z":       "application/x-7z-compressed",
        "906":     "application/x-906",
        "907":     "drawing/907",
        "IVF":     "video/x-ivf",
        "a11":     "application/x-a11",
        "aac":     "audio/x-aac",
        "acp":     "audio/x-mei-aac",
        "ai":      "application/postscript",
        "aif":     "audio/aiff",
        "aifc":    "audio/aiff",
        "aiff":    "audio/aiff",
        "anv":     "application/x-anv",
        "apk":     "application/vnd.android.package-archive",
        "asa":     "text/asa",
        "asf":     "video/x-ms-asf",
        "asp":     "text/asp",
        "asx":     "video/x-ms-asf",
        "atom":    "application/atom+xml",
        "au":      "audio/basic",
        "avi":     "video/avi",
        "awf":     "application/vnd.adobe.workflow",
        "biz":     "text/xml",
        "bmp":     "application/x-bmp",
        "bot":     "application/x-bot",
        "bz2":     "application/x-bzip2",
        "c4t":     "application/x-c4t",
        "c90":     "application/x-c90",
        "cal":     "application/x-cals",
        "cat":     "application/vnd.ms-pki.seccat",
        "cdf":     "application/x-netcdf",
        "cdr":     "application/x-cdr",
        "cel":     "application/x-cel",
        "cer":     "application/x-x509-ca-cert",
        "cg4":     "application/x-g4",
        "cgm":     "application/x-cgm",
        "cit":     "application/x-cit",
        "class":   "java/*",
        "cml":     "text/xml",
        "cmp":     "application/x-cmp",
        "cmx":     "application/x-cmx",
        "cot":     "application/x-cot",
        "crl":     "application/pkix-crl",
        "crt":     "application/x-x509-ca-cert",
        "csi":     "application/x-csi",
        "css":     "text/css",
        "csv":     "text/csv",
        "cu":      "application/cu-seeme",
        "cut":     "application/x-cut",
        "dbf":     "application/x-dbf",
        "dbm":     "application/x-dbm",
        "dbx":     "application/x-dbx",
        "dcd":     "text/xml",
        "dcx":     "application/x-dcx",
        "deb":     "application/x-debian-package",
        "der":     "application/x-x509-ca-cert",
        "dgn":     "application/x-dgn",
        "dib":     "application/x-dib",
        "dll":     "application/x-msdownload",
        "doc":     "application/msword",
        "docx":    "application/vnd.openxmlformats-officedocument.wordprocessingml.document",
        "dot":     "application/msword",
        "drw":     "application/x-drw",
        "dtd":     "text/xml",
        "dvi":     "application/x-dvi",
        "dwf":     "application/x-dwf",
        "dwg":     "application/x-dwg",
        "dxb":     "application/x-dxb",
        "dxf":     "application/x-dxf",
        "edn":     "application/vnd.adobe.edn",
        "emf":     "application/x-emf",
        "eml":     "message/rfc822",
        "ent":     "text/xml",
        "eot":     "application/vnd.ms-fontobject",
        "epi":     "application/x-epi",
        "eps":     "application/postscript",
        "epub":    "application/epub+zip",
        "etd":     "application/x-ebx",
        "etx":     "text/x-setext",
        "exe":     "application/x-msdownload",
        "fax":     "image/fax",
        "fdf":     "application/vnd.fdf",
        "fif":     "application/fractals",
        "flac":    "audio/flac",
        "flv":     "video/x-flv",
        "fo":      "text/xml",
        "frm":     "application/x-frm",
        "g4":      "application/x-g4",
        "gbr":     "application/x-gbr",
        "gif":     "image/gif",
        "gl2":     "application/x-gl2",
        "gp4":     "application/x-gp4",
        "gz":      "application/gzip",
        "hgl":     "application/x-hgl",
        "hmr":     "application/x-hmr",
        "hpg":     "application/x-hpgl",
        "hpl":     "application/x-hpl",
        "hqx":     "application/mac-binhex40",
        "hrf":     "application/x-hrf",
        "hta":     "application/hta",
        "htc":     "text/x-component",
        "htm":     "text/html",
        "html":    "text/html",
        "htt":     "text/webviewhtml",
        "htx":     "text/html",
        "icb":     "application/x-icb",
        "ico":     "application/x-ico",
        "ics":     "text/calendar",
        "iff":     "application/x-iff",
        "ig4":     "application/x-g4",
        "igs":     "application/x-igs",
        "iii":     "application/x-iphone",
        "img":     "application/x-img",
        "ini":     "text/plain",
        "ins":     "application/x-internet-signup",
        "ipa":     "application/vnd.iphone",
        "iso":     "application/x-iso9660-image",
        "isp":     "application/x-internet-signup",
        "jar":     "application/java-archive",
        "java":    "java/*",
        "jfif":    "image/jpeg",
        "jpe":     "image/jpeg",
        "jpeg":    "image/jpeg",
        "jpg":     "image/jpeg",
        "js":      "application/x-javascript",
        "json":    "application/json",
        "jsp":     "text/html",
        "la1":     "audio/x-liquid-file",
        "lar":     "application/x-laplayer-reg",
        "latex":   "application/x-latex",
        "lavs":    "audio/x-liquid-secure",
        "lbm":     "application/x-lbm",
        "lmsff":   "audio/x-la-lms",
        "log":     "text/plain",
        "ls":      "application/x-javascript",
        "ltr":     "application/x-ltr",
        "m1v":     "video/x-mpeg",
        "m2v":     "video/x-mpeg",
        "m3u":     "audio/mpegurl",
        "m4a":     "audio/mp4",
        "m4e":     "video/mpeg4",
        "m4v":     "video/mp4",
        "mac":     "application/x-mac",
        "man":     "application/x-troff-man",
        "math":    "text/xml",
        "mdb":     "application/msaccess",
        "mfp":     "application/x-shockwave-flash",
        "mht":     "message/rfc822",
        "mhtml":   "message/rfc822",
        "mi":      "application/x-mi",
        "mid":     "audio/mid",
        "midi":    "audio/mid",
        "mil":     "application/x-mil",
        "mml":     "text/xml",
        "mnd":     "audio/x-musicnet-download",
        "mns":     "audio/x-musicnet-stream",
        "mocha":   "application/x-javascript",
        "mov":     "video/quicktime",
        "movie":   "video/x-sgi-movie",
        "mp1":     "audio/mp1",
        "mp2":     "audio/mp2",
        "mp2v":    "video/mpeg",
        "mp3":     "audio/mp3",
        "mp4":     "video/mp4",
        "mp4a":    "audio/mp4",
        "mp4v":    "video/mp4",
        "mpa":     "video/x-mpg",
        "mpd":     "application/vnd.ms-project",
        "mpe":     "video/x-mpeg",
        "mpeg":    "video/mpg",
        "mpg":     "video/mpg",
        "mpg4":    "video/mp4",
        "mpga":    "audio/rn-mpeg",
        "mpp":     "application/vnd.ms-project",
        "mps":     "video/x-mpeg",
        "mpt":     "application/vnd.ms-project",
        "mpv":     "video/mpg",
        "mpv2":    "video/mpeg",
        "mpw":     "application/vnd.ms-project",
        "mpx":     "application/vnd.ms-project",
        "mtx":     "text/xml",
        "mxp":     "application/x-mmxp",
        "net":     "image/pnetvue",
        "nrf":     "application/x-nrf",
        "nws":     "message/rfc822",
        "odc":     "text/x-ms-odc",
        "oga":     "audio/ogg",
        "ogg":     "audio/ogg",
        "ogv":     "video/ogg",
        "ogx":     "application/ogg",
        "out":     "application/x-out",
        "p10":     "application/pkcs10",
        "p12":     "application/x-pkcs12",
        "p7b":     "application/x-pkcs7-certificates",
        "p7c":     "application/pkcs7-mime",
        "p7m":     "application/pkcs7-mime",
        "p7r":     "application/x-pkcs7-certreqresp",
        "p7s":     "application/pkcs7-signature",
        "pbm":     "image/x-portable-bitmap",
        "pc5":     "application/x-pc5",
        "pci":     "application/x-pci",
        "pcl":     "application/x-pcl",
        "pcx":     "application/x-pcx",
        "pdf":     "application/pdf",
        "pdx":     "application/vnd.adobe.pdx",
        "pfx":     "application/x-pkcs12",
        "pgl":     "application/x-pgl",
        "pgm":     "image/x-portable-graymap",
        "pic":     "application/x-pic",
        "pko":     "application/vnd.ms-pki.pko",
        "pl":      "application/x-perl",
        "plg":     "text/html",
        "pls":     "audio/scpls",
        "plt":     "application/x-plt",
        "png":     "image/png",
        "pnm":     "image/x-portable-anymap",
        "pot":     "application/vnd.ms-powerpoint",
        "ppa":     "application/vnd.ms-powerpoint",
        "ppm":     "application/x-ppm",
        "pps":     "application/vnd.ms-powerpoint",
        "ppt":     "application/vnd.ms-powerpoint",
        "pptx":    "application/vnd.openxmlformats-officedocument.presentationml.presentation",
        "pr":      "application/x-pr",
        "prf":     "application/pics-rules",
        "prn":     "application/x-prn",
        "prt":     "application/x-prt",
        "ps":      "application/postscript",
        "ptn":     "application/x-ptn",
        "pwz":     "application/vnd.ms-powerpoint",
        "qt":      "video/quicktime",
        "r3t":     "text/vnd.rn-realtext3d",
        "ra":      "audio/vnd.rn-realaudio",
        "ram":     "audio/x-pn-realaudio",
        "rar":     "application/x-rar-compressed",
        "ras":     "application/x-ras",
        "rat":     "application/rat-file",
        "rdf":     "text/xml",
        "rec":     "application/vnd.rn-recording",
        "red":     "application/x-red",
        "rgb":     "application/x-rgb",
        "rjs":     "application/vnd.rn-realsystem-rjs",
        "rjt":     "application/vnd.rn-realsystem-rjt",
        "rlc":     "application/x-rlc",
        "rle":     "application/x-rle",
        "rm":      "application/vnd.rn-realmedia",
        "rmf":     "application/vnd.adobe.rmf",
        "rmi":     "audio/mid",
        "rmj":     "application/vnd.rn-realsystem-rmj",
        "rmm":     "audio/x-pn-realaudio",
        "rmp":     "application/vnd.rn-rn_music_package",
        "rms":     "application/vnd.rn-realmedia-secure",
        "rmvb":    "application/vnd.rn-realmedia-vbr",
        "rmx":     "application/vnd.rn-realsystem-rmx",
        "rnx":     "application/vnd.rn-realplayer",
        "rp":      "image/vnd.rn-realpix",
        "rpm":     "audio/x-pn-realaudio-plugin",
        "rsml":    "application/vnd.rn-rsml",
        "rss":     "application/rss+xml",
        "rt":      "text/vnd.rn-realtext",
        "rtf":     "application/x-rtf",
        "rv":      "video/vnd.rn-realvideo",
        "sam":     "application/x-sam",
        "sat":     "application/x-sat",
        "sdp":     "application/sdp",
        "sdw":     "application/x-sdw",
        "sgm":     "text/sgml",
        "sgml":    "text/sgml",
        "sis":     "application/vnd.symbian.install",
        "sisx":    "application/vnd.symbian.install",
        "sit":     "application/x-stuffit",
        "slb":     "application/x-slb",
        "sld":     "application/x-sld",
        "slk":     "drawing/x-slk",
        "smi":     "application/smil",
        "smil":    "application/smil",
        "smk":     "application/x-smk",
        "snd":     "audio/basic",
        "sol":     "text/plain",
        "sor":     "text/plain",
        "spc":     "application/x-pkcs7-certificates",
        "spl":     "application/futuresplash",
        "spp":     "text/xml",
        "ssm":     "application/streamingmedia",
        "sst":     "application/vnd.ms-pki.certstore",
        "stl":     "application/vnd.ms-pki.stl",
        "stm":     "text/html",
        "sty":     "application/x-sty",
        "svg":     "text/xml",
        "swf":     "application/x-shockwave-flash",
        "tar":     "application/x-tar",
        "tdf":     "application/x-tdf",
        "tg4":     "application/x-tg4",
        "tga":     "application/x-tga",
        "tif":     "image/tiff",
        "tiff":    "image/tiff",
        "tld":     "text/xml",
        "top":     "drawing/x-top",
        "torrent": "application/x-bittorrent",
        "tsd":     "text/xml",
        "ttf":     "application/x-font-ttf",
        "txt":     "text/plain",
        "uin":     "application/x-icq",
        "uls":     "text/iuls",
        "vcf":     "text/x-vcard",
        "vda":     "application/x-vda",
        "vdx":     "application/vnd.visio",
        "vml":     "text/xml",
        "vpg":     "application/x-vpeg005",
        "vsd":     "application/vnd.visio",
        "vss":     "application/vnd.visio",
        "vst":     "application/x-vst",
        "vsw":     "application/vnd.visio",
        "vsx":     "application/vnd.visio",
        "vtx":     "application/vnd.visio",
        "vxml":    "text/xml",
        "wav":     "audio/wav",
        "wax":     "audio/x-ms-wax",
        "wb1":     "application/x-wb1",
        "wb2":     "application/x-wb2",
        "wb3":     "application/x-wb3",
        "wbmp":    "image/vnd.wap.wbmp",
        "webm":    "video/webm",
        "wiz":     "application/msword",
        "wk3":     "application/x-wk3",
        "wk4":     "application/x-wk4",
        "wkq":     "application/x-wkq",
        "wks":     "application/x-wks",
        "wm":      "video/x-ms-wm",
        "wma":     "audio/x-ms-wma",
        "wmd":     "application/x-ms-wmd",
        "wmf":     "application/x-wmf",
        "wml":     "text/vnd.wap.wml",
        "wmv":     "video/x-ms-wmv",
        "wmx":     "video/x-ms-wmx",
        "wmz":     "application/x-ms-wmz",
        "woff":    "application/x-font-woff",
        "wp6":     "application/x-wp6",
        "wpd":     "application/x-wpd",
        "wpg":     "application/x-wpg",
        "wpl":     "application/vnd.ms-wpl",
        "wq1":     "application/x-wq1",
        "wr1":     "application/x-wr1",
        "wri":     "application/x-wri",
        "wrk":     "application/x-wrk",
        "ws":      "application/x-ws",
        "ws2":     "application/x-ws",
        "wsc":     "text/scriptlet",
        "wsdl":    "text/xml",
        "wvx":     "video/x-ms-wvx",
        "x_b":     "application/x-x_b",
        "x_t":     "application/x-x_t",
        "xap":     "application/x-silverlight-app",
        "xbm":     "image/x-xbitmap",
        "xdp":     "application/vnd.adobe.xdp",
        "xdr":     "text/xml",
        "xfd":     "application/vnd.adobe.xfd",
        "xfdf":    "application/vnd.adobe.xfdf",
        "xhtml":   "text/html",
        "xls":     "application/vnd.ms-excel",
        "xlsx":    "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet",
        "xlw":     "application/x-xlw",
        "xml":     "text/xml",
        "xpl":     "audio/scpls",
        "xpm":     "image/x-xpixmap",
        "xq":      "text/xml",
        "xql":     "text/xml",
        "xquery":  "text/xml",
        "xsd":     "text/xml",
        "xsl":     "text/xml",
        "xslt":    "text/xml",
        "xwd":     "application/x-xwd",
        "yaml":    "text/yaml",
        "yml":     "text/yaml",
        "zip":     "application/zip",
    };
    var key = str.toString();
    var begin = key && key.lastIndexOf('.');
    if (begin && begin >= 0) {
        key = key.substr(begin + 1);
        if (key) {
            var value = contentTypes[key.toLowerCase()];
            if (value) {
                return value;
            } else {
                return "application/octet-stream";
            }
        } else {
            return "application/octet-stream";
        }
    } else {
        return "application/octet-stream";
    }
};

/*
 下载文件夹
 */
TaskControlAction.prototype.downloadPath = function(bucket, parentPath,prefix, marker, path,num,cb) {
    var self = this;
    if(!self.iteratorsObject){
        return false;
    }
    if(!num){
        num = 0;
    }
    if(parentPath.charAt(parentPath.length-1)!='/'){
        parentPath += "/";
    }
    if(path.charAt(path.length-1)!='/'){
        path += "/";
    }
    //由于是异步，所以如果之前执行了abortall，则不再加入
    if(!this.taskManager.abortAll){
        if(global.shareDownload){
            var params = {Bucket: bucket, Prefix: prefix,Policy:global.Policy,Signature:global.Signature,AWSAccessKeyId:global.AccessKeyId, Marker: marker};
            s3.listObjectsByUrl(params, function (err, data) {
                if (err) {
                    //超时错误重试
                    if (err.hasOwnProperty('code') && err.code == 'NetworkingError') {
                        if (num < 4) {
                            num++;
                            setTimeout(function () {
                                self.downloadPath(bucket, parentPath, prefix, marker, path, num, cb);
                                global.logger.error('this is downloadPath function error = ', num);
                            }, num * 60000);
                        } else {
                            if (cb) {
                                cb(err);
                            }
                        }
                    } else {
                        //不是超时的错误直接返回
                        cb(err);
                    }
                    global.logger.error("download share folder ",err,params);
                }
                else {
                    if (data.IsTruncated){
                        num = 0;
                        async.mapSeries(data.Contents, function(object, callback) {
                            setTimeout(function() {
                                var key = object.Key;
                                if (key[key.length - 1] != '/') {
                                    var subKey = key.substring(parentPath.length-1, key.length);
                                    self.downloadFile(bucket, key, path + subKey, object.Size);
                                }else{
                                    var subKey = key.substring(parentPath.length-1, key.length);
                                    self.downloadFile(bucket, key, path + subKey, 0);
                                }
                                if(self.taskManager.abortAll)
                                    callback("abort by user");
                                else
                                    callback();
                            }, intervalTime);
                        }, function(err,results) {
                            if(err){
                                global.logger.error(err);
                            }
                        });
                        self.downloadPath(bucket, parentPath, prefix, data.NextMarker, path,num,cb);
                    }
                    else{
                        /**
                         * 顺序执行，一个完了才执行下一个。
                         */
                        async.mapSeries(data.Contents, function(object, callback) {
                            setTimeout(function() {
                                var key = object.Key;
                                if (key[key.length - 1] != '/') {
                                    var subKey = key.substring(parentPath.length-1, key.length);
                                    self.downloadFile(bucket, key, path + subKey, object.Size);
                                }else{
                                    var subKey = key.substring(parentPath.length-1, key.length);
                                    self.downloadFile(bucket, key, path + subKey, 0);
                                }
                                if(self.taskManager.abortAll)
                                    callback("abort by user");
                                else
                                    callback();
                            }, intervalTime);
                        }, function(err,results) {
                            if(err){
                                global.logger.error(err);
                            }
                        });
                    }
                }
            });
        }else{
            var params = {Bucket: bucket, Prefix: prefix, Marker: marker, MaxKeys: 100};
            s3.listObjects(params, function (err, data) {
                if (err) {
                    //超时错误重试
                    if (err.hasOwnProperty('code') && err.code == 'NetworkingError') {
                        if (num < 4) {
                            num++;
                            setTimeout(function () {
                                self.downloadPath(bucket, parentPath, prefix, marker, path, num, cb);
                                global.logger.error('this is downloadPath function error = ', num);
                            }, num * 60000);
                        } else {
                            if (cb) {
                                cb(err);
                            }
                        }
                    } else {
                        //不是超时的错误直接返回
                        cb(err);
                    }
                    global.logger.error(err);
                }
                else {
                    if (data.IsTruncated){
                        num = 0;
                        async.mapSeries(data.Contents, function(object, callback) {
                            setTimeout(function() {
                                var key = object.Key;
                                if (key[key.length - 1] != '/') {
                                    var subKey = key.substring(parentPath.length-1, key.length);
                                    self.downloadFile(bucket, key, path + subKey, object.Size);
                                }else{
                                    var subKey = key.substring(parentPath.length-1, key.length);
                                    self.downloadFile(bucket, key, path + subKey, 0);
                                }
                                if(self.taskManager.abortAll)
                                    callback("abort by user");
                                else
                                    callback();
                            }, intervalTime);
                        }, function(err,results) {
                            if(err){
                                global.logger.error(err);
                            }
                        });
                        self.downloadPath(bucket, parentPath, prefix, data.NextMarker, path,num,cb);
                    }
                    else{
                        /**
                         * 顺序执行，一个完了才执行下一个。
                         */
                        async.mapSeries(data.Contents, function(object, callback) {
                            setTimeout(function() {
                                var key = object.Key;
                                if (key[key.length - 1] != '/') {
                                    var subKey = key.substring(parentPath.length-1, key.length);
                                    self.downloadFile(bucket, key, path + subKey, object.Size);
                                }else{
                                    var subKey = key.substring(parentPath.length-1, key.length);
                                    self.downloadFile(bucket, key, path + subKey, 0);
                                }
                                if(self.taskManager.abortAll)
                                    callback("abort by user");
                                else
                                    callback();
                            }, intervalTime);
                        }, function(err,results) {
                            if(err){
                                global.logger.error(err);
                            }
                        });
                    }
                }
            });
        }
    }
};


//开始任务
TaskControlAction.prototype.startTask = function (id) {
    this.taskManager.start(id);
};
//开始所有任务
TaskControlAction.prototype.startAllTasks = function (callback) {
    this.taskManager.startAll(callback);
};

//暂停任务
TaskControlAction.prototype.pauseTask = function (id) {
    this.taskManager.pause(id);
};

//暂停所有任务
TaskControlAction.prototype.pauseAllTasks = function (callback) {
    var self = this;

    if(self.taskManager){
        //暂停任务
        self.taskManager.setPauseAll(true);

        //暂停运行中的任务
        var tasks = self.taskManager.runningMap;
        for (var id in tasks) {
            self.taskManager.pause(id);
        }

        self.taskManager.pauseAllTask(callback);

    }else{
        if(callback){
            callback();
        }
    }
};

//取消任务
TaskControlAction.prototype.abortTask = function (id, callback) {
    this.taskManager.abort(id);
    this.taskManager.on("onAbort",function(data){
        if(callback){
            callback(data)
        }
    });
};

//取消所有任务
TaskControlAction.prototype.abortAllTasks = function (callback) {
    this.taskManager.setAbortAll(true);
    var self = this;
    for(var i = 0;i<self.iterators.length;i++){
        if(self.iterators[i]){
            self.iterators[i].setAbort();
        }
    }
    self.iterators=[];

    self.fileQueue = [];
    var tasks = self.taskManager.runningMap;
    for (var id in tasks) {
        self.taskManager.abort(id);
    }
    //取消等待、暂停和失败任务
    self.taskManager.abortAllTasks(callback);

};

//切换用户时停止递归遍历UDS对象方法
TaskControlAction.prototype.updateIteratorsObject = function (stat) {
    this.iteratorsObject = stat;
};

/**
 * 检查数据库数据是否需要清理，如果超过50W则清理部分数据
 * @param num 每次查询条数
 * @param total 总共清理多少条记录
 */
TaskControlAction.prototype.checkDBData = function (){
    var self = this;
    self.taskManager.countTasks({stat:"SUCCESS"}, function (err,data) {
        global.logger.warn("Current completed task num: " + data);
        if(data && data > Const("NUM_MONGODB_TASKS")){
            global.logger.warn("Completed task num=" + data + ", over than maxinum（"+Const("NUM_MONGODB_TASKS")+"), start to execute clearDBData()");
            self.clearDBData();
        }
    });
};

/**
 * 清理数据库数据
 * @param num 每次查询条数
 * @param total 总共清理多少条记录
 */
TaskControlAction.prototype.clearDBData = function (){
    var self = this;
    var isStill = true;//是否继续递归
    var exePath=process.execPath;//文件路径
    exePath = exePath.replace(/\\/g, "/");
    exePath = exePath.substring(0,exePath.lastIndexOf('/')+1);
    var date = new Date().toISOString().replace(/:/g,"").replace(/-/g,"");
    date = date.substring(0,date.lastIndexOf('.'));

    //每次都新生成一个csv文件
    var writeStream = fs.createWriteStream(exePath+'history/historyDBData'+date+'.csv',{encoding:'utf8'});
    var headData = new Buffer("Name,Type,BucketName,Size,StartTime,endTime,\n",'utf8');
    writeStream.write(new Buffer('\xEF\xBB\xBF','binary'));
    writeStream.write(headData);
    var delNum = 0;//已删除任务个数

    //递归执行清理操作，每次从数据库取num条数据
    async.whilst(
        function() {
            return isStill && (delNum < Const("NUM_CLEAR_NUM"));
        },
        function(cb) {
            var query={"stat":"SUCCESS"};
            self.taskManager.listTasks(query,Const("NUM_MONGODB_LIMITS"),1,function(err,data){
                if(data && data.length > 0){
                    delNum += data.length;
                    //将任务备份到csv文件中
                    commonUtils.writeTaskData2CSV(writeStream,data, function () {
                        var ids = [];
                        for(var x in data){
                            ids.push(data[x].id);
                        }
                        var startTime = new Date().getTime();
                        self.taskManager.deleteTasks(ids, function () {
                            console.log('times:',new Date().getTime() - startTime);
                            cb();
                        });
                    });
                }else{
                    isStill = false;
                    cb();
                }
            });
        },
        function(err) {
            // 3s have passed
            global.logger.info('Clear dbdata success!');
            writeStream.end();
        }
    );
};

/*
  添加恢复任务
 */
TaskControlAction.prototype.recoverObject = function(bucket, key, size,stat,errCode,days,tier) {
    var self = this;
    var uuid = guid();
    var task =
    {
        id: uuid,
        type: "recover",
        bucket: bucket,
        size: size || 0 ,
        progress: "0%",
        stat: stat,
        key: key,
        days: days,
        tier: tier
    };
    if(errCode){
        task.errMessage = errCode;
        task.endTime = new Date();
    }
    self.taskManager.push(task);
};

//恢复并下载
TaskControlAction.prototype.recoverAndDownload = function(bucket, key, size,stat,filePath,errCode,days,tier,etag) {
    var self = this;
    var uuid = guid();
    var task =
        {
            id: uuid,
            type: "recoverAndDownload",
            bucket: bucket,
            size: size || 0 ,
            progress: "0%",
            stat: stat,
            key: key,
            days: days,
            tier: tier,
            path: filePath,
            etag: etag
        };
    if(errCode){
        task.errMessage = errCode;
        task.endTime = new Date();
    }
    self.taskManager.push(task);
};
TaskControlAction.prototype.pasteObject = function(targetBucket, targetKey, sourceBucket, sourceKey, size, action,sourceFsStatus, stat, errCode) {
    var self = this;
    var uuid = guid();
    var task = {
        id: uuid,
        type: action,
        bucket: targetBucket,
        size: size,
        stat: stat,
        key: targetKey,
        sourceBucket: sourceBucket,
        sourceKey: sourceKey,
        modifyTime: new Date(),
        startTime: new Date(),
        fsStatus: global.curFileInterface,
        sourceFsStatus: sourceFsStatus
    };
    global.logger.info("push task lingnan:", task);
    if(errCode){
        task.errMessage = errCode;
    }
    self.taskManager.push(task);
}

module.exports = TaskControlAction;